#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type
from ansible.plugins.action import ActionBase

try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator,
    )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
import re
try:
    import ipaddress
except ImportError:
    IPADDRESS_INSTALLED = False
else:
    IPADDRESS_INSTALLED = True
from ansible_collections.cisco.ise.plugins.plugin_utils.ise import (
    ISESDK,
    ise_argument_spec,
    ise_compare_equality,
    get_dict_result,
)
from ansible_collections.cisco.ise.plugins.plugin_utils.exceptions import (
    InconsistentParameters,
)

# Get common arguments specification
argument_spec = ise_argument_spec()
# Add arguments specific for this module
argument_spec.update(dict(
    state=dict(type="str", default="present", choices=["present", "absent"]),
    subnet=dict(type="str"),
    domains=dict(type="str"),
    sgt=dict(type="str"),
    vn=dict(type="str"),
    id=dict(type="str"),
))

required_if = [
    ("state", "present", ["id"], True),
    ("state", "absent", ["id"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class FilterPolicy(object):
    def __init__(self, params, ise):
        self.ise = ise
        self.new_object = dict(
            subnet=params.get("subnet"),
            domains=params.get("domains"),
            sgt=params.get("sgt"),
            vn=params.get("vn"),
            id=params.get("id"),
        )

    def is_same_subnet(self, new, current):
        if IPADDRESS_INSTALLED:
            new_net = None
            current_net = None
            try:
                new_net = ipaddress.ip_network(new, strict=False)
            except ValueError:
                new_net = None
            try:
                current_net = ipaddress.ip_network(current, strict=False)
            except ValueError:
                current_net = None
            if new_net and current_net:
                conflict = current_net.overlaps(new_net) or new_net.overlaps(current_net)
                # conflict = current_net.subnet_of(new_net) or new_net.subnet_of(current_net)
                # They are the mostly the same, both have overlapping net
                return conflict
            elif new_net is None and current_net is None:
                return True
            else:
                return False
        else:
            if new and current:
                return new == current
            else:
                return not current and not new

    def get_sgt_by_name(self, name):
        if not name:
            return None
        try:
            gen_items_responses = self.ise.exec(
                family="filter_policy",
                function="get_filter_policy_generator"
            )
            for items_response in gen_items_responses:
                items = items_response.response['SearchResult']['resources']
                result = get_dict_result(items, 'name', name)
                if result:
                    return result
        except (TypeError, AttributeError) as e:
            self.ise.fail_json(
                msg=(
                    "An error occured when executing operation."
                    " Check the configuration of your API Settings and API Gateway settings on your ISE server."
                    " This collection assumes that the API Gateway, the ERS APIs and OpenAPIs are enabled."
                    " You may want to enable the (ise_debug: True) argument."
                    " The error was: {error}"
                ).format(error=e)
            )
        except Exception:
            result = None
        return result

    def get_sgt_by_id(self, id):
        if not id:
            return None
        try:
            result = self.ise.exec(
                family="sgt",
                function="get_security_group_by_id",
                params={"id": id},
                handle_func_exception=False,
            ).response['Sgt']
        except (TypeError, AttributeError) as e:
            self.ise.fail_json(
                msg=(
                    "An error occured when executing operation."
                    " Check the configuration of your API Settings and API Gateway settings on your ISE server."
                    " This collection assumes that the API Gateway, the ERS APIs and OpenAPIs are enabled."
                    " You may want to enable the (ise_debug: True) argument."
                    " The error was: {error}"
                ).format(error=e)
            )
        except Exception:
            result = None
        return result

    def is_same_sgt(self, new, current):
        # Values can be id or name
        def clean_excess(name):
            if name:
                return re.sub(r"\s*\(.*\)$", "", name)
            else:
                return name
        has_new = self.get_sgt_by_id(new) or self.get_sgt_by_name(clean_excess(new))
        has_current = self.get_sgt_by_id(current) or self.get_sgt_by_name(clean_excess(current))
        if has_new and has_current:
            return has_new.get("id") == has_current.get("id")
        else:
            return not has_current and not has_new

    def is_same_vn(self, new, current):
        if new and current:
            return new == current
        else:
            return not current and not new

    def get_object_by_name(self, name, new_subnet, new_sgt, new_vn):
        # NOTICE: Does not have a get by name method or it is in another action
        result = None
        gen_items_responses = self.ise.exec(
            family="filter_policy",
            function="get_filter_policy_generator"
        )
        try:
            for items_response in gen_items_responses:
                items = items_response.response['SearchResult']['resources']
                for item in items:
                    current = self.get_object_by_id(item.get('id'))
                    if current:
                        has_same_subnet = self.is_same_subnet(new_subnet, current.get('subnet'))
                        has_same_sgt = self.is_same_sgt(new_sgt, current.get('sgt'))
                        has_same_vn = self.is_same_vn(new_vn, current.get('vn'))
                    if has_same_subnet and has_same_sgt and has_same_vn:
                        result = dict(current)
                        return result
        except (TypeError, AttributeError) as e:
            self.ise.fail_json(
                msg=(
                    "An error occured when executing operation."
                    " Check the configuration of your API Settings and API Gateway settings on your ISE server."
                    " This collection assumes that the API Gateway, the ERS APIs and OpenAPIs are enabled."
                    " You may want to enable the (ise_debug: True) argument."
                    " The error was: {error}"
                ).format(error=e)
            )
        except Exception:
            result = None
            return result
        return result

    def get_object_by_id(self, id):
        try:
            result = self.ise.exec(
                family="filter_policy",
                function="get_filter_policy_by_id",
                handle_func_exception=False,
                params={"id": id}
            ).response['ERSFilterPolicy']
        except (TypeError, AttributeError) as e:
            self.ise.fail_json(
                msg=(
                    "An error occured when executing operation."
                    " Check the configuration of your API Settings and API Gateway settings on your ISE server."
                    " This collection assumes that the API Gateway, the ERS APIs and OpenAPIs are enabled."
                    " You may want to enable the (ise_debug: True) argument."
                    " The error was: {error}"
                ).format(error=e)
            )
        except Exception:
            result = None
        return result

    def exists(self):
        prev_obj = None
        id_exists = False
        name_exists = False
        o_id = self.new_object.get("id")
        id_exists = o_id and self.get_object_by_id(o_id)
        if id_exists:
            prev_obj = self.get_object_by_id(o_id)
        if not id_exists:
            name = self.new_object.get("name")
            subnet = self.new_object.get("subnet")
            sgt = self.new_object.get("sgt")
            vn = self.new_object.get("vn")
            prev_obj = self.get_object_by_name(name, subnet, sgt, vn)
            name_exists = prev_obj is not None and isinstance(prev_obj, dict)
            if name_exists:
                id_ = prev_obj.get("id")
                self.new_object.update(dict(id=id_))
        it_exists = prev_obj is not None and isinstance(prev_obj, dict)
        return (it_exists, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object

        obj_params = [
            ("subnet", "subnet"),
            ("domains", "domains"),
            ("sgt", "sgt"),
            ("vn", "vn"),
            ("id", "id"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (ISE) params
        # If any does not have eq params, it requires update
        return any(not ise_compare_equality(current_obj.get(ise_param),
                                            requested_obj.get(ansible_param))
                   for (ise_param, ansible_param) in obj_params)

    def create(self):
        result = self.ise.exec(
            family="filter_policy",
            function="create_filter_policy",
            params=self.new_object,
        ).response
        return result

    def update(self):
        result = self.ise.exec(
            family="filter_policy",
            function="update_filter_policy_by_id",
            params=self.new_object
        ).response
        return result

    def delete(self):
        result = self.ise.exec(
            family="filter_policy",
            function="delete_filter_policy_by_id",
            params=self.new_object
        ).response
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail("ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        ise = ISESDK(params=self._task.args)
        obj = FilterPolicy(self._task.args, ise)

        state = self._task.args.get("state")

        response = None

        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    ise_update_response = obj.update()
                    self._result.update(dict(ise_update_response=ise_update_response))
                    (obj_exists, updated_obj) = obj.exists()
                    response = updated_obj
                    ise.object_updated()
                else:
                    response = prev_obj
                    ise.object_already_present()
            else:
                ise_create_response = obj.create()
                (obj_exists, created_obj) = obj.exists()
                response = created_obj
                ise.object_created()

        elif state == "absent":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                obj.delete()
                response = prev_obj
                ise.object_deleted()
            else:
                ise.object_already_absent()

        self._result.update(dict(ise_response=response))
        self._result.update(ise.exit_json())
        return self._result



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

try:
    import lxml.etree
except ImportError:
    # should be handled in module importing this one
    pass


HOSTTECH_WSDL_DEFAULT_ENTRIES = [
    (125, 42, 'A', '', '1.2.3.4', 3600, None, None),
    (126, 42, 'A', '*', '1.2.3.5', 3600, None, None),
    (127, 42, 'AAAA', '', '2001:1:2::3', 3600, None, None),
    (128, 42, 'AAAA', '*', '2001:1:2::4', 3600, None, None),
    (129, 42, 'MX', '', 'example.com', 3600, None, '10'),
    (130, 42, 'NS', '', 'ns3.hostserv.eu', 10800, None, None),
    (131, 42, 'NS', '', 'ns2.hostserv.eu', 10800, None, None),
    (132, 42, 'NS', '', 'ns1.hostserv.eu', 10800, None, None),
]

HOSTTECH_JSON_DEFAULT_ENTRIES = [
    # (125, 42, 'A', '', '1.2.3.4', 3600, None, None),
    {
        'id': 125,
        'type': 'A',
        'name': '',
        'ipv4': '1.2.3.4',
        'ttl': 3600,
        'comment': '',
    },
    # (126, 42, 'A', '*', '1.2.3.5', 3600, None, None),
    {
        'id': 126,
        'type': 'A',
        'name': '*',
        'ipv4': '1.2.3.5',
        'ttl': 3600,
        'comment': '',
    },
    # (127, 42, 'AAAA', '', '2001:1:2::3', 3600, None, None),
    {
        'id': 127,
        'type': 'AAAA',
        'name': '',
        'ipv6': '2001:1:2::3',
        'ttl': 3600,
        'comment': '',
    },
    # (128, 42, 'AAAA', '*', '2001:1:2::4', 3600, None, None),
    {
        'id': 128,
        'type': 'AAAA',
        'name': '*',
        'ipv6': '2001:1:2::4',
        'ttl': 3600,
        'comment': '',
    },
    # (129, 42, 'MX', '', 'example.com', 3600, None, '10'),
    {
        'id': 129,
        'type': 'MX',
        'ownername': '',
        'name': 'example.com',
        'pref': 10,
        'ttl': 3600,
        'comment': '',
    },
    # (130, 42, 'NS', '', 'ns3.hostserv.eu', 10800, None, None),
    {
        'id': 130,
        'type': 'NS',
        'ownername': '',
        'targetname': 'ns3.hostserv.eu',
        'ttl': 10800,
        'comment': '',
    },
    # (131, 42, 'NS', '', 'ns2.hostserv.eu', 10800, None, None),
    {
        'id': 131,
        'type': 'NS',
        'ownername': '',
        'targetname': 'ns2.hostserv.eu',
        'ttl': 10800,
        'comment': '',
    },
    # (132, 42, 'NS', '', 'ns1.hostserv.eu', 10800, None, None),
    {
        'id': 132,
        'type': 'NS',
        'ownername': '',
        'targetname': 'ns1.hostserv.eu',
        'ttl': 10800,
        'comment': '',
    },
]


def validate_wsdl_call(conditions):
    def predicate(content):
        assert content.startswith(b"<?xml version='1.0' encoding='utf-8'?>\n")

        root = lxml.etree.fromstring(content)
        header = None
        body = None

        for header_ in root.iter(lxml.etree.QName('http://schemas.xmlsoap.org/soap/envelope/', 'Header').text):
            header = header_
        for body_ in root.iter(lxml.etree.QName('http://schemas.xmlsoap.org/soap/envelope/', 'Body').text):
            body = body_

        for condition in conditions:
            if not condition(content, header, body):
                return False
        return True

    return predicate


def get_wsdl_value(root, name):
    for auth in root.iter(name):
        return auth
    raise Exception('Cannot find child "{0}" in node {1}: {2}'.format(name, root, lxml.etree.tostring(root)))


def expect_wsdl_authentication(username, password):
    def predicate(content, header, body):
        auth = get_wsdl_value(header, lxml.etree.QName('auth', 'authenticate').text)
        assert get_wsdl_value(auth, 'UserName').text == username
        assert get_wsdl_value(auth, 'Password').text == password
        return True

    return predicate


def check_wsdl_nil(node):
    nil_flag = node.get(lxml.etree.QName('http://www.w3.org/2001/XMLSchema-instance', 'nil'))
    if nil_flag != 'true':
        print(nil_flag)
    assert nil_flag == 'true'


def check_wsdl_value(node, value, type=None):
    if type is not None:
        type_text = node.get(lxml.etree.QName('http://www.w3.org/2001/XMLSchema-instance', 'type'))
        assert type_text is not None, 'Cannot find type in {0}: {1}'.format(node, lxml.etree.tostring(node))
        i = type_text.find(':')
        if i < 0:
            ns = None
        else:
            ns = node.nsmap.get(type_text[:i])
            type_text = type_text[i + 1:]
        if ns != type[0] or type_text != type[1]:
            print(ns, type[0], type_text, type[1])
        assert ns == type[0] and type_text == type[1]
    if node.text != value:
        print(node.text, value)
    assert node.text == value


def find_xml_map_entry(map_root, key_name, allow_non_existing=False):
    for map_entry in map_root.iter('item'):
        key = get_wsdl_value(map_entry, 'key')
        value = get_wsdl_value(map_entry, 'value')
        if key.text == key_name:
            check_wsdl_value(key, key_name, type=('http://www.w3.org/2001/XMLSchema', 'string'))
            return value
    if allow_non_existing:
        return None
    raise Exception('Cannot find map entry with key "{0}" in node {1}: {2}'.format(key_name, map_root, lxml.etree.tostring(map_root)))


def expect_wsdl_value(path, value, type=None):
    def predicate(content, header, body):
        node = body
        for entry in path:
            node = get_wsdl_value(node, entry)
        check_wsdl_value(node, value, type=type)
        return True

    return predicate


def add_wsdl_answer_start_lines(lines):
    lines.extend([
        '<?xml version="1.0" encoding="UTF-8"?>\n',
        '<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"'
        ' xmlns:ns1="https://ns1.hosttech.eu/public/api"'
        ' xmlns:xsd="http://www.w3.org/2001/XMLSchema"'
        ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"'
        ' xmlns:ns2="http://xml.apache.org/xml-soap"'
        ' xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"'
        ' SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">',
        '<SOAP-ENV:Header>',
        '<ns1:authenticateResponse>',
        '<return xsi:type="xsd:boolean">true</return>',
        '</ns1:authenticateResponse>',
        '</SOAP-ENV:Header>',
        '<SOAP-ENV:Body>',
    ])


def add_wsdl_answer_end_lines(lines):
    lines.extend([
        '</SOAP-ENV:Body>',
        '</SOAP-ENV:Envelope>'
    ])


def add_wsdl_dns_record_lines(lines, entry, tag_name):
    lines.extend([
        '<{tag_name} xsi:type="ns2:Map">'.format(tag_name=tag_name),
        '<item><key xsi:type="xsd:string">id</key><value xsi:type="xsd:int">{value}</value></item>'.format(value=entry[0]),
        '<item><key xsi:type="xsd:string">zone</key><value xsi:type="xsd:int">{value}</value></item>'.format(value=entry[1]),
        '<item><key xsi:type="xsd:string">type</key><value xsi:type="xsd:string">{value}</value></item>'.format(value=entry[2]),
        '<item><key xsi:type="xsd:string">prefix</key><value xsi:type="xsd:string">{value}</value></item>'.format(value=entry[3]),
        '<item><key xsi:type="xsd:string">target</key><value xsi:type="xsd:string">{value}</value></item>'.format(value=entry[4]),
        '<item><key xsi:type="xsd:string">ttl</key><value xsi:type="xsd:int">{value}</value></item>'.format(value=entry[5]),
    ])
    if entry[6] is None:
        lines.append('<item><key xsi:type="xsd:string">comment</key><value xsi:nil="true"/></item>')
    else:
        lines.append('<item><key xsi:type="xsd:string">comment</key><value xsi:type="xsd:string">{value}</value></item>'.format(value=entry[6]))
    if entry[7] is None:
        lines.append('<item><key xsi:type="xsd:string">priority</key><value xsi:nil="true"/></item>')
    else:
        lines.append('<item><key xsi:type="xsd:string">priority</key><value xsi:type="xsd:int">{value}</value></item>'.format(value=entry[7]))
    lines.append('</{tag_name}>'.format(tag_name=tag_name))


def create_wsdl_zones_answer(zone_id, zone_name, entries):
    lines = []
    add_wsdl_answer_start_lines(lines)
    lines.extend([
        '<ns1:getZoneResponse>',
        '<return xsi:type="ns2:Map">',
        '<item><key xsi:type="xsd:string">id</key><value xsi:type="xsd:int">{zone_id}</value></item>'.format(zone_id=zone_id),
        '<item><key xsi:type="xsd:string">user</key><value xsi:type="xsd:int">23</value></item>',
        '<item><key xsi:type="xsd:string">name</key><value xsi:type="xsd:string">{zone_name}</value></item>'.format(zone_name=zone_name),
        '<item><key xsi:type="xsd:string">email</key><value xsi:type="xsd:string">dns@hosttech.eu</value></item>',
        '<item><key xsi:type="xsd:string">ttl</key><value xsi:type="xsd:int">10800</value></item>',
        '<item><key xsi:type="xsd:string">nameserver</key><value xsi:type="xsd:string">ns1.hostserv.eu</value></item>',
        '<item><key xsi:type="xsd:string">serial</key><value xsi:type="xsd:string">12345</value></item>',
        '<item><key xsi:type="xsd:string">serialLastUpdate</key><value xsi:type="xsd:int">0</value></item>',
        '<item><key xsi:type="xsd:string">refresh</key><value xsi:type="xsd:int">7200</value></item>',
        '<item><key xsi:type="xsd:string">retry</key><value xsi:type="xsd:int">120</value></item>',
        '<item><key xsi:type="xsd:string">expire</key><value xsi:type="xsd:int">1234567</value></item>',
        '<item><key xsi:type="xsd:string">template</key><value xsi:nil="true"/></item>',
        '<item><key xsi:type="xsd:string">ns3</key><value xsi:type="xsd:int">1</value></item>',
    ])
    lines.append(
        '<item><key xsi:type="xsd:string">records</key><value SOAP-ENC:arrayType="ns2:Map[{count}]" xsi:type="SOAP-ENC:Array">'.format(
            count=len(entries)))
    for entry in entries:
        add_wsdl_dns_record_lines(lines, entry, 'item')
    lines.extend([
        '</value>',
        '</item>',
        '</return>',
        '</ns1:getZoneResponse>',
    ])
    add_wsdl_answer_end_lines(lines)
    return ''.join(lines)


def create_wsdl_zone_not_found_answer():
    lines = [
        '<?xml version="1.0" encoding="UTF-8"?>\n',
        '<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"'
        ' xmlns:ns1="https://ns1.hosttech.eu/public/api"'
        ' xmlns:xsd="http://www.w3.org/2001/XMLSchema"'
        ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"'
        ' xmlns:ns2="http://xml.apache.org/xml-soap"'
        ' xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/"'
        ' SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">',
        '<SOAP-ENV:Header>',
        '<ns1:authenticateResponse>',
        '<return xsi:type="xsd:boolean">true</return>',
        '</ns1:authenticateResponse>',
        '</SOAP-ENV:Header>',
        '<SOAP-ENV:Fault>',
        '<faultstring>zone not found</faultstring>'
        '</SOAP-ENV:Fault>',
        '</SOAP-ENV:Envelope>'
    ]
    return ''.join(lines)


def check_wsdl_record(record_data, entry):
    check_wsdl_value(find_xml_map_entry(record_data, 'type'), entry[2], type=('http://www.w3.org/2001/XMLSchema', 'string'))
    prefix = find_xml_map_entry(record_data, 'prefix')
    if entry[3]:
        check_wsdl_value(prefix, entry[3], type=('http://www.w3.org/2001/XMLSchema', 'string'))
    elif prefix is not None:
        check_wsdl_nil(prefix)
    check_wsdl_value(find_xml_map_entry(record_data, 'target'), entry[4], type=('http://www.w3.org/2001/XMLSchema', 'string'))
    check_wsdl_value(find_xml_map_entry(record_data, 'ttl'), str(entry[5]), type=('http://www.w3.org/2001/XMLSchema', 'int'))
    if entry[6] is None:
        comment = find_xml_map_entry(record_data, 'comment', allow_non_existing=True)
        if comment is not None:
            check_wsdl_nil(comment)
    else:
        check_wsdl_value(find_xml_map_entry(record_data, 'comment'), entry[6], type=('http://www.w3.org/2001/XMLSchema', 'string'))
    if entry[7] is None:
        check_wsdl_nil(find_xml_map_entry(record_data, 'priority'))
    else:
        check_wsdl_value(find_xml_map_entry(record_data, 'priority'), entry[7], type=('http://www.w3.org/2001/XMLSchema', 'string'))


def validate_wsdl_add_request(zone, entry):
    def predicate(content, header, body):
        fn_data = get_wsdl_value(body, lxml.etree.QName('https://ns1.hosttech.eu/public/api', 'addRecord').text)
        check_wsdl_value(get_wsdl_value(fn_data, 'search'), zone, type=('http://www.w3.org/2001/XMLSchema', 'string'))
        check_wsdl_record(get_wsdl_value(fn_data, 'recorddata'), entry)
        return True

    return predicate


def validate_wsdl_update_request(entry):
    def predicate(content, header, body):
        fn_data = get_wsdl_value(body, lxml.etree.QName('https://ns1.hosttech.eu/public/api', 'updateRecord').text)
        check_wsdl_value(get_wsdl_value(fn_data, 'recordId'), str(entry[0]), type=('http://www.w3.org/2001/XMLSchema', 'int'))
        check_wsdl_record(get_wsdl_value(fn_data, 'recorddata'), entry)
        return True

    return predicate


def validate_wsdl_del_request(entry):
    def predicate(content, header, body):
        fn_data = get_wsdl_value(body, lxml.etree.QName('https://ns1.hosttech.eu/public/api', 'deleteRecord').text)
        check_wsdl_value(get_wsdl_value(fn_data, 'recordId'), str(entry[0]), type=('http://www.w3.org/2001/XMLSchema', 'int'))
        return True

    return predicate


def create_wsdl_add_result(entry):
    lines = []
    add_wsdl_answer_start_lines(lines)
    lines.append('<ns1:addRecordResponse>')
    add_wsdl_dns_record_lines(lines, entry, 'return')
    lines.append('</ns1:addRecordResponse>')
    add_wsdl_answer_end_lines(lines)
    return ''.join(lines)


def create_wsdl_update_result(entry):
    lines = []
    add_wsdl_answer_start_lines(lines)
    lines.append('<ns1:updateRecordResponse>')
    add_wsdl_dns_record_lines(lines, entry, 'return')
    lines.append('</ns1:updateRecordResponse>')
    add_wsdl_answer_end_lines(lines)
    return ''.join(lines)


def create_wsdl_del_result(success):
    lines = []
    add_wsdl_answer_start_lines(lines)
    lines.extend([
        '<ns1:deleteRecordResponse>',
        '<return xsi:type="xsd:boolean">{success}</return>'.format(success='true' if success else 'false'),
        '</ns1:deleteRecordResponse>',
    ])
    add_wsdl_answer_end_lines(lines)
    return ''.join(lines)


HOSTTECH_WSDL_DEFAULT_ZONE_RESULT = create_wsdl_zones_answer(42, 'example.com', HOSTTECH_WSDL_DEFAULT_ENTRIES)

HOSTTECH_WSDL_ZONE_NOT_FOUND = create_wsdl_zone_not_found_answer()

HOSTTECH_JSON_ZONE_LIST_RESULT = {
    "data": [
        {
            "id": 42,
            "name": "example.com",
            "email": "test@example.com",
            "ttl": 10800,
            "nameserver": "ns1.hosttech.ch",
            "dnssec": False,
        },
        {
            "id": 43,
            "name": "foo.com",
            "email": "test@foo.com",
            "ttl": 10800,
            "nameserver": "ns1.hosttech.ch",
            'dnssec': True,
            'dnssec_email': 'test@foo.com',
        },
    ],
}

HOSTTECH_JSON_ZONE_GET_RESULT = {
    "data": {
        "id": 42,
        "name": "example.com",
        "email": "test@example.com",
        "ttl": 10800,
        "nameserver": "ns1.hosttech.ch",
        "dnssec": False,
        "records": HOSTTECH_JSON_DEFAULT_ENTRIES,
    }
}

HOSTTECH_JSON_ZONE_2_GET_RESULT = {
    "data": {
        "id": 43,
        "name": "foo.com",
        "email": "test@foo.com",
        "ttl": 10800,
        "nameserver": "ns1.hosttech.ch",
        'dnssec': True,
        'dnssec_email': 'test@foo.com',
        'ds_records': [
            {
                'key_tag': 12345,
                'algorithm': 8,
                'digest_type': 1,
                'digest': '012356789ABCDEF0123456789ABCDEF012345678',
                'flags': 257,
                'protocol': 3,
                'public_key':
                    'MuhdzsQdqEGShwjtJDKZZjdKqUSGluFzTTinpuEeIRzLLcgkwgAPKWFa '
                    'eQntNlmcNDeCziGwpdvhJnvKXEMbFcZwsaDIJuWqERxAQNGABWfPlCLh '
                    'HQPnbpRPNKipSdBaUhuOubvFvjBpFAwiwSAapRDVsAgKvjXucfXpFfYb '
                    'pCundbAXBWhbpHVbqgmGoixXzFSwUsGVYLPpBCiDlLJwzjRKYYaoVYge '
                    'kMtKFYUVnWIKbectWkDFdVqXwkKigCUDiuTTJxOBRJRNzGiDNMWBjYSm '
                    'bBCAHMaMYaghLbYTwyKXltdHTHwBwtswGNfpnEdSpKFzZJonBZArQfHD '
                    'lfceKgmKwEF=',
            },
            {
                'key_tag': 12345,
                'algorithm': 8,
                'digest_type': 2,
                'digest': '0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF',
                'flags': 257,
                'protocol': 3,
                'public_key':
                    'MuhdzsQdqEGShwjtJDKZZjdKqUSGluFzTTinpuEeIRzLLcgkwgAPKWFa '
                    'eQntNlmcNDeCziGwpdvhJnvKXEMbFcZwsaDIJuWqERxAQNGABWfPlCLh '
                    'HQPnbpRPNKipSdBaUhuOubvFvjBpFAwiwSAapRDVsAgKvjXucfXpFfYb '
                    'pCundbAXBWhbpHVbqgmGoixXzFSwUsGVYLPpBCiDlLJwzjRKYYaoVYge '
                    'kMtKFYUVnWIKbectWkDFdVqXwkKigCUDiuTTJxOBRJRNzGiDNMWBjYSm '
                    'bBCAHMaMYaghLbYTwyKXltdHTHwBwtswGNfpnEdSpKFzZJonBZArQfHD '
                    'lfceKgmKwEF=',
            }
        ],
        "records": [],
    }
}

HOSTTECH_JSON_ZONE_RECORDS_GET_RESULT = {
    "data": HOSTTECH_JSON_DEFAULT_ENTRIES,
}



from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import pytest
from dateutil import parser as date_parser

from ansible_collections.amazon.aws.plugins.module_utils.core import normalize_boto3_result

example_date_txt = '2020-12-30T00:00:00.000Z'
example_date_iso = '2020-12-30T00:00:00+00:00'
example_date = date_parser.parse(example_date_txt)


normalize_boto3_result_data = [
    (dict(),
     dict()
     ),
    # Bool
    (dict(param1=False),
     dict(param1=False)
     ),
    # Simple string (shouldn't be touched
    (dict(date_example=example_date_txt),
     dict(date_example=example_date_txt)
     ),
    (dict(date_example=example_date_iso),
     dict(date_example=example_date_iso)
     ),
    # Datetime -> String
    (dict(date_example=example_date),
     dict(date_example=example_date_iso)
     ),
    (list(),
     list()
     ),
    (list([False]),
     list([False])
     ),
    (list([example_date_txt]),
     list([example_date_txt])
     ),
    (list([example_date_iso]),
     list([example_date_iso])
     ),
    (list([example_date]),
     list([example_date_iso])
     ),
]


@pytest.mark.parametrize("input_params, output_params", normalize_boto3_result_data)
def test_normalize_boto3_result(input_params, output_params):

    assert normalize_boto3_result(input_params) == output_params



# -*- coding: utf-8 -*-

# Copyright (c) 2020-2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type

from ansible_collections.community.dns.plugins.plugin_utils.public_suffix import PUBLIC_SUFFIX_LIST


def _remove_suffix(dns_name, suffix, keep_trailing_period):
    suffix_len = len(suffix)
    if suffix_len and suffix_len < len(dns_name) and not keep_trailing_period:
        suffix_len += 1
    return dns_name[:-suffix_len] if suffix_len else dns_name


def get_registrable_domain(dns_name,
                           keep_unknown_suffix=True,
                           only_if_registerable=True,
                           normalize_result=False,
                           icann_only=False):
    '''Given DNS name, returns the registrable domain.'''
    return PUBLIC_SUFFIX_LIST.get_registrable_domain(
        dns_name,
        keep_unknown_suffix=keep_unknown_suffix,
        only_if_registerable=only_if_registerable,
        normalize_result=normalize_result,
        icann_only=icann_only,
    )


def get_public_suffix(dns_name,
                      keep_leading_period=True,
                      keep_unknown_suffix=True,
                      normalize_result=False,
                      icann_only=False):
    '''Given DNS name, returns the public suffix.'''
    suffix = PUBLIC_SUFFIX_LIST.get_suffix(
        dns_name,
        keep_unknown_suffix=keep_unknown_suffix,
        normalize_result=normalize_result,
        icann_only=icann_only,
    )
    if suffix and len(suffix) < len(dns_name) and keep_leading_period:
        suffix = '.' + suffix
    return suffix


def remove_registrable_domain(dns_name,
                              keep_trailing_period=False,
                              keep_unknown_suffix=True,
                              only_if_registerable=True,
                              icann_only=False):
    '''Given DNS name, returns the part before the registrable_domain.'''
    suffix = PUBLIC_SUFFIX_LIST.get_registrable_domain(
        dns_name,
        keep_unknown_suffix=keep_unknown_suffix,
        only_if_registerable=only_if_registerable,
        normalize_result=False,
        icann_only=icann_only,
    )
    return _remove_suffix(dns_name, suffix, keep_trailing_period)


def remove_public_suffix(dns_name,
                         keep_trailing_period=False,
                         keep_unknown_suffix=True,
                         icann_only=False):
    '''Given DNS name, returns the part before the public suffix.'''
    suffix = PUBLIC_SUFFIX_LIST.get_suffix(
        dns_name,
        keep_unknown_suffix=keep_unknown_suffix,
        normalize_result=False,
        icann_only=icann_only,
    )
    return _remove_suffix(dns_name, suffix, keep_trailing_period)


class FilterModule(object):
    '''Ansible jinja2 filters'''

    def filters(self):
        return {
            'get_public_suffix': get_public_suffix,
            'get_registrable_domain': get_registrable_domain,
            'remove_public_suffix': remove_public_suffix,
            'remove_registrable_domain': remove_registrable_domain,
        }



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: wafv2_ip_set_info
version_added: 1.5.0
author:
  - "Markus Bergholz (@markuman)"
short_description: Get information about wafv2 ip sets
description:
  - Get information about existing wafv2 ip sets.
options:
    name:
      description:
        - The name of the IP set.
      required: true
      type: str
    scope:
      description:
        - Specifies whether this is for an AWS CloudFront distribution or for a regional application.
      choices: ["CLOUDFRONT","REGIONAL"]
      required: true
      type: str

extends_documentation_fragment:
- amazon.aws.aws
- amazon.aws.ec2

'''

EXAMPLES = '''
- name: test ip set
  wafv2_ip_set_info:
    name: test02
    scope: REGIONAL
'''

RETURN = """
addresses:
  description: Current addresses of the ip set
  sample:
    - 8.8.8.8/32
    - 8.8.4.4/32
  returned: Always, as long as the ip set exists
  type: list
arn:
  description: IP set arn
  sample: "arn:aws:wafv2:eu-central-1:11111111:regional/ipset/test02/4b007330-2934-4dc5-af24-82dcb3aeb127"
  type: str
  returned: Always, as long as the ip set exists
description:
  description: Description of the ip set
  sample: Some IP set description
  returned: Always, as long as the ip set exists
  type: str
ip_address_version:
  description: IP version of the ip set
  sample: IPV4
  type: str
  returned: Always, as long as the ip set exists
name:
  description: IP set name
  sample: test02
  returned: Always, as long as the ip set exists
  type: str
"""

try:
    from botocore.exceptions import ClientError, BotoCoreError
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.core import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import camel_dict_to_snake_dict


def list_ip_sets(wafv2, scope, fail_json_aws, Nextmarker=None):
    # there is currently no paginator for wafv2
    req_obj = {
        'Scope': scope,
        'Limit': 100
    }
    if Nextmarker:
        req_obj['NextMarker'] = Nextmarker

    try:
        response = wafv2.list_ip_sets(**req_obj)
        if response.get('NextMarker'):
            response['IPSets'] += list_ip_sets(wafv2, scope, fail_json_aws, Nextmarker=response.get('NextMarker')).get('IPSets')
    except (BotoCoreError, ClientError) as e:
        fail_json_aws(e, msg="Failed to list wafv2 ip set.")
    return response


def get_ip_set(wafv2, name, scope, id, fail_json_aws):
    try:
        response = wafv2.get_ip_set(
            Name=name,
            Scope=scope,
            Id=id
        )
    except (BotoCoreError, ClientError) as e:
        fail_json_aws(e, msg="Failed to get wafv2 ip set.")
    return response


def main():

    arg_spec = dict(
        name=dict(type='str', required=True),
        scope=dict(type='str', required=True, choices=['CLOUDFRONT', 'REGIONAL'])
    )

    module = AnsibleAWSModule(
        argument_spec=arg_spec,
        supports_check_mode=True,
    )

    name = module.params.get("name")
    scope = module.params.get("scope")

    wafv2 = module.client('wafv2')

    # check if ip set exist
    response = list_ip_sets(wafv2, scope, module.fail_json_aws)

    id = None

    for item in response.get('IPSets'):
        if item.get('Name') == name:
            id = item.get('Id')

    retval = {}
    existing_set = None
    if id:
        existing_set = get_ip_set(wafv2, name, scope, id, module.fail_json_aws)
        retval = camel_dict_to_snake_dict(existing_set.get('IPSet'))

    module.exit_json(**retval)


if __name__ == '__main__':
    main()



# (c) 2021 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish

from __future__ import absolute_import, division, print_function

from typing import Sequence


__metaclass__ = type

from textwrap import dedent

from ansible_collections.cisco.nxos.plugins.modules import nxos_logging_global
from ansible_collections.cisco.nxos.tests.unit.compat.mock import patch
from ansible_collections.cisco.nxos.tests.unit.modules.utils import AnsibleFailJson

from .nxos_module import TestNxosModule, set_module_args


ignore_provider_arg = True


class TestNxosLoggingGlobalModule(TestNxosModule):

    module = nxos_logging_global

    def setUp(self):
        super(TestNxosLoggingGlobalModule, self).setUp()

        self.mock_get_resource_connection = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module_base.get_resource_connection",
        )
        self.get_resource_connection = self.mock_get_resource_connection.start()

        self.mock_get_config = patch(
            "ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.logging_global.logging_global.Logging_globalFacts.get_config",
        )
        self.get_config = self.mock_get_config.start()

    def tearDown(self):
        super(TestNxosLoggingGlobalModule, self).tearDown()
        self.get_resource_connection.stop()
        self.get_config.stop()

    def test_nxos_logging_global_linear_merged(self):
        # test merged for linear attributes
        self.get_config.return_value = dedent(
            """\
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    console=dict(severity="alert"),
                    module=dict(severity="notification"),
                    monitor=dict(severity="critical"),
                    history=dict(severity="informational", size=4096),
                    rate_limit="disabled",
                    rfc_strict=True,
                    origin_id=dict(string="nodeA"),
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        commands = [
            "logging console 1",
            "logging module 5",
            "logging monitor 2",
            "logging history 6",
            "logging history size 4096",
            "no logging rate-limit",
            "logging rfc-strict 5424",
            "logging origin-id string nodeA",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_linear_merged_idempotent(self):
        # test merged for linear attributes (idempotent)
        self.get_config.return_value = dedent(
            """\
            logging console 1
            logging module 5
            logging monitor 2
            logging history 6
            logging history size 4096
            no logging rate-limit
            logging rfc-strict 5424
            logging origin-id string nodeA
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    console=dict(severity="alert"),
                    module=dict(severity="notification"),
                    monitor=dict(severity="critical"),
                    history=dict(severity="informational", size=4096),
                    rate_limit="disabled",
                    rfc_strict=True,
                    origin_id=dict(string="nodeA"),
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_logging_global_linear_merged_2(self):
        # test merged for linear attributes - 2
        self.get_config.return_value = dedent(
            """\
            logging console 1
            no logging module
            logging monitor 2
            logging history 6
            logging history size 4096
            no logging rate-limit
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    console=dict(state="disabled"),
                    module=dict(state="enabled"),
                    monitor=dict(state="disabled"),
                    history=dict(severity="informational", size=4096),
                    rate_limit="enabled",
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        commands = [
            "no logging console 1",
            "logging module",
            "no logging monitor 2",
            "logging rate-limit",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_linear_replaced(self):
        # test replaced for linear attributes
        self.get_config.return_value = dedent(
            """\
            logging console 1
            logging module 5
            logging monitor 2
            logging history 6
            logging history size 4096
            no logging rate-limit
            logging rfc-strict 5424
            logging origin-id string nodeA
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    console=dict(severity="notification"),
                    monitor=dict(severity="critical"),
                    history=dict(size=4096),
                ),
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "logging console 5",
            "logging module",
            "no logging history 6",
            "logging rate-limit",
            "no logging rfc-strict 5424",
            "no logging origin-id string nodeA",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_linear_replaced(self):
        # test replaced for linear attributes
        self.get_config.return_value = dedent(
            """\
            logging console 1
            logging module 5
            logging monitor 2
            logging history 6
            logging history size 4096
            no logging rate-limit
            logging rfc-strict 5424
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    console=dict(severity="notification"),
                    monitor=dict(severity="critical"),
                    history=dict(size=4096),
                ),
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "logging console 5",
            "logging module",
            "no logging history 6",
            "logging rate-limit",
            "no logging rfc-strict 5424",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_linear_merged_3(self):
        # test merged for linear attributes - 3
        self.get_config.return_value = dedent(
            """\
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    origin_id=dict(hostname=True),
                    ip=dict(
                        access_list=dict(
                            cache=dict(entries=16384, interval=200, threshold=80),
                            detailed=True,
                            include=dict(sgt=True),
                        ),
                    ),
                    source_interface="Ethernet1/100",
                    timestamp="milliseconds",
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        commands = [
            "logging origin-id hostname",
            "logging ip access-list cache entries 16384",
            "logging ip access-list cache interval 200",
            "logging ip access-list cache threshold 80",
            "logging ip access-list detailed",
            "logging ip access-list include sgt",
            "logging source-interface Ethernet1/100",
            "logging timestamp milliseconds",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_linear_merged_3_idempotent(self):
        # test merged for linear attributes - 3 (idempotent)
        self.get_config.return_value = dedent(
            """\
            logging origin-id hostname
            logging ip access-list cache entries 16384
            logging ip access-list cache interval 200
            logging ip access-list cache threshold 80
            logging ip access-list detailed
            logging ip access-list include sgt
            logging source-interface Ethernet1/100
            logging timestamp milliseconds
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    origin_id=dict(hostname=True),
                    ip=dict(
                        access_list=dict(
                            cache=dict(entries=16384, interval=200, threshold=80),
                            detailed=True,
                            include=dict(sgt=True),
                        ),
                    ),
                    source_interface="Ethernet1/100",
                    timestamp="milliseconds",
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_logging_global_linear_replaced_3(self):
        # test replaced for linear attributes - 3
        self.get_config.return_value = dedent(
            """\
            logging origin-id hostname
            logging ip access-list cache entries 16384
            logging ip access-list cache interval 200
            logging ip access-list cache threshold 80
            logging ip access-list detailed
            logging ip access-list include sgt
            logging source-interface Ethernet1/100
            logging timestamp milliseconds
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    origin_id=dict(ip="192.168.1.1"),
                    ip=dict(access_list=dict(cache=dict(entries=16384, interval=210))),
                    source_interface="Ethernet1/120",
                ),
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "no logging origin-id hostname",
            "logging origin-id ip 192.168.1.1",
            "logging ip access-list cache interval 210",
            "no logging ip access-list cache threshold 80",
            "no logging ip access-list detailed",
            "no logging ip access-list include sgt",
            "logging source-interface Ethernet1/120",
            "no logging timestamp milliseconds",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_complex_merged(self):
        # test merged for complex attributes
        self.get_config.return_value = dedent(
            """\
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    logfile=dict(
                        name="nodeA_log",
                        size=4096,
                        severity="critical",
                        persistent_threshold=80,
                    ),
                    facilities=[
                        dict(facility="auth", severity="alert"),
                        dict(facility="ospfv3", severity="critical"),
                        dict(facility="cron", severity="notification"),
                        dict(facility="vlan_mgr", severity="notification"),
                    ],
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        commands = [
            "logging logfile nodeA_log 2 size 4096 persistent threshold 80",
            "logging level cron 5",
            "logging level ospfv3 2",
            "logging level auth 1",
            "logging level vlan_mgr 5",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_complex_merged_idempotent(self):
        # test merged for complex attributes (idempotent)
        self.get_config.return_value = dedent(
            """\
            logging logfile nodeA_log 2 size 4096 persistent threshold 80
            logging level cron 5
            logging level ospfv3 2
            logging level auth 1
            logging level vlan_mgr 5
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    logfile=dict(
                        name="nodeA_log",
                        size=4096,
                        severity="critical",
                        persistent_threshold=80,
                    ),
                    facilities=[
                        dict(facility="auth", severity="alert"),
                        dict(facility="ospfv3", severity="critical"),
                        dict(facility="cron", severity="notification"),
                        dict(facility="vlan_mgr", severity="notification"),
                    ],
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_logging_global_complex_replaced(self):
        # test replaced for complex attributes
        self.get_config.return_value = dedent(
            """\
            logging logfile nodeA_log 2 size 4096 persistent threshold 80
            logging level cron 5
            logging level ospfv3 2
            logging level auth 1
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    facilities=[
                        dict(facility="auth", severity="alert"),
                        dict(facility="ospfv3", severity="critical"),
                        dict(facility="ospf", severity="notification"),
                    ],
                ),
                state="replaced",
            ),
            ignore_provider_arg,
        )

        commands = [
            "logging logfile messages 5",
            "no logging level cron 5",
            "logging level ospf 5",
        ]

        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_event_merged(self):
        # test merged for `event`
        self.get_config.return_value = dedent(
            """\
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    event=dict(
                        link_status=dict(enable=False, default=False),
                        trunk_status=dict(enable=False, default=True),
                    ),
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        commands = [
            "no logging event link-status enable",
            "no logging event link-status default",
            "no logging event trunk-status enable",
            "logging event trunk-status default",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_event_merged(self):
        # test merged for `event`
        self.get_config.return_value = dedent(
            """\
            no logging event link-status enable
            no logging event link-status default
            no logging event trunk-status enable
            logging event trunk-status default
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    event=dict(
                        link_status=dict(enable=False, default=False),
                        trunk_status=dict(enable=False, default=True),
                    ),
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_logging_global_event_replaced(self):
        # test replaced for `event`
        self.get_config.return_value = dedent(
            """\
            no logging event link-status enable
            no logging event link-status default
            no logging event trunk-status enable
            logging event trunk-status default
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    event=dict(
                        link_status=dict(default=False),
                        trunk_status=dict(enable=False),
                    ),
                ),
                state="replaced",
            ),
            ignore_provider_arg,
        )

        commands = [
            "logging event link-status enable",
            "no logging event trunk-status default",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_gathered_empty(self):
        set_module_args(dict(running_config="", state="gathered"), ignore_provider_arg)
        result = self.execute_module(changed=False)
        self.assertEqual(result["gathered"], {})

    def test_nxos_logging_global_gathered(self):
        # test gathered
        self.get_config.return_value = dedent(
            """\
            logging console 1
            logging module 5
            logging monitor 2
            logging history 6
            logging level vlan_mgr 5
            logging history size 4096
            no logging rate-limit
            logging rfc-strict 5424
            logging origin-id string nodeA
            """,
        )
        set_module_args(dict(state="gathered"), ignore_provider_arg)
        gathered = {
            "console": {"severity": "alert"},
            "module": {"severity": "notification"},
            "monitor": {"severity": "critical"},
            "history": {"severity": "informational", "size": 4096},
            "rate_limit": "disabled",
            "rfc_strict": True,
            "origin_id": {"string": "nodeA"},
            "facilities": [{"facility": "vlan_mgr", "severity": "notification"}],
        }
        result = self.execute_module(changed=False)
        self.assertEqual(result["gathered"], gathered)

    def test_nxos_logging_global_parsed(self):
        # test parsed
        cfg = dedent(
            """\
            logging console 1
            logging module 5
            logging monitor 2
            logging history 6
            logging history size 4096
            no logging rate-limit
            logging rfc-strict 5424
            logging origin-id string nodeA
            """,
        )
        set_module_args(dict(running_config=cfg, state="parsed"), ignore_provider_arg)
        parsed = {
            "console": {"severity": "alert"},
            "module": {"severity": "notification"},
            "monitor": {"severity": "critical"},
            "history": {"severity": "informational", "size": 4096},
            "rate_limit": "disabled",
            "rfc_strict": True,
            "origin_id": {"string": "nodeA"},
        }
        result = self.execute_module(changed=False)
        self.assertEqual(result["parsed"], parsed)

    def test_nxos_logging_global_hosts_merged(self):
        # test merged for `hosts`
        self.get_config.return_value = dedent(
            """\
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    hosts=[
                        dict(
                            host="192.168.1.1",
                            severity="alert",
                            facility="auth",
                            port=5891,
                            use_vrf="default",
                            secure=dict(trustpoint=dict(client_identity="test")),
                        ),
                        dict(host="192.168.1.2"),
                        dict(host="192.168.1.3", severity="critical"),
                    ],
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        commands = [
            "logging server 192.168.1.1 1 port 5891 secure trustpoint client-identity test facility auth use-vrf default",
            "logging server 192.168.1.2",
            "logging server 192.168.1.3 2",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_hosts_replaced(self):
        # test replaced for `hosts`
        self.get_config.return_value = dedent(
            """\
            logging server 192.168.1.1 1 port 5891 secure trustpoint client-identity test facility auth use-vrf default
            logging server 192.168.1.2
            logging server 192.168.1.3 2
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    hosts=[
                        dict(
                            host="192.168.1.1",
                            severity="alert",
                            facility="auth",
                            port=5891,
                            use_vrf="default",
                            secure=dict(trustpoint=dict(client_identity="test")),
                        ),
                        dict(
                            host="192.168.1.3",
                            severity="debugging",
                            use_vrf="management",
                        ),
                    ],
                ),
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "logging server 192.168.1.1 1 port 5891 secure trustpoint client-identity test facility auth use-vrf default",
            "no logging server 192.168.1.2",
            "no logging server 192.168.1.3 2",
            "logging server 192.168.1.3 7 use-vrf management",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_linear_negated_merged(self):
        # test merged for negated linear attributes
        self.get_config.return_value = dedent(
            """\
            no logging console
            no logging module
            no logging monitor
            """,
        )
        set_module_args(
            dict(
                config=dict(
                    console=dict(severity="notification"),
                    module=dict(state="enabled"),
                    monitor=dict(severity="critical"),
                ),
                state="merged",
            ),
            ignore_provider_arg,
        )
        commands = ["logging console 5", "logging module", "logging monitor 2"]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_deleted(self):
        # test deleted
        self.get_config.return_value = dedent(
            """\
            logging console 1
            logging module 5
            logging monitor 2
            logging history 6
            logging history size 4096
            no logging rate-limit
            logging rfc-strict 5424
            logging origin-id string nodeA
            """,
        )
        set_module_args(dict(state="deleted"), ignore_provider_arg)
        commands = [
            "logging console",
            "logging module",
            "logging monitor",
            "no logging history 6",
            "no logging history size 4096",
            "logging rate-limit",
            "no logging rfc-strict 5424",
            "no logging origin-id string nodeA",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_event_replaced_2(self):
        # test replaced for `event` - 2
        self.get_config.return_value = dedent(
            """\
            no logging event link-status enable
            no logging event link-status default
            no logging event trunk-status enable
            logging event trunk-status default
            """,
        )
        set_module_args(
            dict(
                config=dict(event=dict(trunk_status=dict(enable=False, default=True))),
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "logging event link-status enable",
            "logging event link-status default",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))

    def test_nxos_logging_global_event_replaced_2(self):
        # test replaced for `event` - 2
        self.get_config.return_value = dedent(
            """\
            no logging event link-status enable
            no logging event link-status default
            no logging event trunk-status enable
            logging event trunk-status default
            """,
        )
        set_module_args(
            dict(
                config=dict(console=dict(severity="critical")),
                state="replaced",
            ),
            ignore_provider_arg,
        )
        commands = [
            "logging event link-status enable",
            "logging event link-status default",
            "logging event trunk-status enable",
            "no logging event trunk-status default",
            "logging console 2",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(set(result["commands"]), set(commands))



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage Check Point Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_access_rules
short_description: Manages access-rules objects on Check Point over Web Services API
description:
  - Manages access-rules objects on Check Point devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
version_added: "2.9"
author: "Shiran Golzar (@chkp-shirango)"
options:
  layer:
    description:
      - Layer that the rule belongs to identified by the name or UID.
    type: str
    required: true
  rules:
    description:
      - List of rules.
    type: list
    required: true
    suboptions:
      name:
        description:
          - Object name.
        type: str
        required: True
      action:
        description:
          - a "Accept", "Drop", "Ask", "Inform", "Reject", "User Auth", "Client Auth", "Apply Layer".
        type: str
      action_settings:
        description:
          - Action settings.
        type: dict
        suboptions:
          enable_identity_captive_portal:
            description:
              - N/A
            type: bool
          limit:
            description:
              - N/A
            type: str
      content:
        description:
          - List of processed file types that this rule applies on.
        type: list
      content_direction:
        description:
          - On which direction the file types processing is applied.
        type: str
        choices: ['any', 'up', 'down']
      content_negate:
        description:
          - True if negate is set for data.
        type: bool
      custom_fields:
        description:
          - Custom fields.
        type: dict
        suboptions:
          field_1:
            description:
              - First custom field.
            type: str
          field_2:
            description:
              - Second custom field.
            type: str
          field_3:
            description:
              - Third custom field.
            type: str
      destination:
        description:
          - Collection of Network objects identified by the name or UID.
        type: list
      destination_negate:
        description:
          - True if negate is set for destination.
        type: bool
      enabled:
        description:
          - Enable/Disable the rule.
        type: bool
      inline_layer:
        description:
          - Inline Layer identified by the name or UID. Relevant only if "Action" was set to "Apply Layer".
        type: str
      install_on:
        description:
          - Which Gateways identified by the name or UID to install the policy on.
        type: list
      service:
        description:
          - Collection of Network objects identified by the name or UID.
        type: list
      service_negate:
        description:
          - True if negate is set for service.
        type: bool
      source:
        description:
          - Collection of Network objects identified by the name or UID.
        type: list
      source_negate:
        description:
          - True if negate is set for source.
        type: bool
      time:
        description:
          - List of time objects. For example, "Weekend", "Off-Work", "Every-Day".
        type: list
      track:
        description:
          - Track Settings.
        type: dict
        suboptions:
          accounting:
            description:
              - Turns accounting for track on and off.
            type: bool
          alert:
            description:
              - Type of alert for the track.
            type: str
            choices: ['none', 'alert', 'snmp', 'mail', 'user alert 1', 'user alert 2', 'user alert 3']
          enable_firewall_session:
            description:
              - Determine whether to generate session log to firewall only connections.
            type: bool
          per_connection:
            description:
              - Determines whether to perform the log per connection.
            type: bool
          per_session:
            description:
              - Determines whether to perform the log per session.
            type: bool
          type:
            description:
              - a "Log", "Extended Log", "Detailed  Log", "None".
            type: str
      user_check:
        description:
          - User check settings.
        type: dict
        suboptions:
          confirm:
            description:
              - N/A
            type: str
            choices: ['per rule', 'per category', 'per application/site', 'per data type']
          custom_frequency:
            description:
              - N/A
            type: dict
            suboptions:
              every:
                description:
                  - N/A
                type: int
              unit:
                description:
                  - N/A
                type: str
                choices: ['hours', 'days', 'weeks', 'months']
          frequency:
            description:
              - N/A
            type: str
            choices: ['once a day', 'once a week', 'once a month', 'custom frequency...']
          interaction:
            description:
              - N/A
            type: str
      vpn:
        description:
          - Communities or Directional.
        type: list
        suboptions:
          community:
            description:
              - List of community name or UID.
            type: list
          directional:
            description:
              - Communities directional match condition.
            type: list
            suboptions:
              from:
                description:
                  - From community name or UID.
                type: str
              to:
                description:
                  - To community name or UID.
                type: str
      comments:
        description:
          - Comments string.
        type: str
      details_level:
        description:
          - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
            representation of the object.
        type: str
        choices: ['uid', 'standard', 'full']
      ignore_warnings:
        description:
          - Apply changes ignoring warnings.
        type: bool
      ignore_errors:
        description:
          - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
        type: bool
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
extends_documentation_fragment: check_point.mgmt.checkpoint_objects_action_module
"""

EXAMPLES = """
- name: add-access-rules
  cp_mgmt_access_rules:
    rules:
      - name: Rule 1
        service:
        - SMTP
        - AOL
        state: present
      - name: Rule 2
        service:
        - SMTP
        state: present
    layer: Network
    auto_publish_session: true
"""

RETURN = """
cp_mgmt_access_rules:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import Connection
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import \
    checkpoint_argument_spec_for_action_module


def main():
    argument_spec = dict(
        rules=dict(type='list', required=True, options=dict(
            name=dict(type='str', required=True),
            action=dict(type='str'),
            action_settings=dict(type='dict', options=dict(
                enable_identity_captive_portal=dict(type='bool'),
                limit=dict(type='str')
            )),
            content=dict(type='list'),
            content_direction=dict(type='str', choices=['any', 'up', 'down']),
            content_negate=dict(type='bool'),
            custom_fields=dict(type='dict', options=dict(
                field_1=dict(type='str'),
                field_2=dict(type='str'),
                field_3=dict(type='str')
            )),
            destination=dict(type='list'),
            destination_negate=dict(type='bool'),
            enabled=dict(type='bool'),
            inline_layer=dict(type='str'),
            install_on=dict(type='list'),
            service=dict(type='list'),
            service_negate=dict(type='bool'),
            source=dict(type='list'),
            source_negate=dict(type='bool'),
            time=dict(type='list'),
            track=dict(type='dict', options=dict(
                accounting=dict(type='bool'),
                alert=dict(type='str',
                           choices=['none', 'alert', 'snmp', 'mail', 'user alert 1', 'user alert 2', 'user alert 3']),
                enable_firewall_session=dict(type='bool'),
                per_connection=dict(type='bool'),
                per_session=dict(type='bool'),
                type=dict(type='str')
            )),
            user_check=dict(type='dict', options=dict(
                confirm=dict(type='str', choices=['per rule', 'per category', 'per application/site', 'per data type']),
                custom_frequency=dict(type='dict', options=dict(
                    every=dict(type='int'),
                    unit=dict(type='str', choices=['hours', 'days', 'weeks', 'months'])
                )),
                frequency=dict(type='str',
                               choices=['once a day', 'once a week', 'once a month', 'custom frequency...']),
                interaction=dict(type='str')
            )),
            vpn=dict(type='list', options=dict(
                community=dict(type='list'),
                directional=dict(type='list', options=dict(
                    to=dict(type='str')
                ))
            )),
            comments=dict(type='str'),
            details_level=dict(type='str', choices=['uid', 'standard', 'full']),
            ignore_warnings=dict(type='bool'),
            ignore_errors=dict(type='bool')
        )),
        layer=dict(type='str', required=True),
        details_level=dict(type='str', choices=['uid', 'standard', 'full'])
    )

    argument_spec['rules']['options']['vpn']['options']['directional']['options']['from'] = dict(type='str')
    argument_spec.update(checkpoint_argument_spec_for_action_module)

    module = AnsibleModule(argument_spec=argument_spec)

    module.exit_json()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for ios_l3_interfaces
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
module: ios_l3_interfaces
short_description: Resource module to configure L3 interfaces.
description:
- This module provides declarative management of Layer-3 interface on Cisco IOS devices.
version_added: 1.0.0
author:
- Sagar Paul (@KB-perByte)
- Sumit Jaiswal (@justjais)
notes:
  - Tested against Cisco IOSv Version 15.6.
  - Using deleted state without config will delete all l3 attributes from all the interfaces.
  - This module works with connection C(network_cli).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
options:
  config:
    description: A dictionary of Layer-3 interface options
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Full name of the interface excluding any logical unit number, i.e. GigabitEthernet0/1.
        type: str
        required: true
      ipv4:
        description:
        - IPv4 address to be set for the Layer-3 interface mentioned in I(name) option.
          The address format is <ipv4 address>/<mask>, the mask is number in range
          0-32 eg. 192.168.0.1/24.
        type: list
        elements: dict
        suboptions:
          address:
            description:
            - Configures the IPv4 address for Interface.
            type: str
          secondary:
            description:
            - Configures the IP address as a secondary address.
            type: bool
          dhcp_client:
            description:
            - Configures and specifies client-id to use over DHCP ip. Note, This option
              shall work only when dhcp is configured as IP.
            - GigabitEthernet interface number
            - This option is DEPRECATED and is replaced with dhcp which
              accepts dict as input this attribute will be removed after 2023-08-01.
            type: str
          dhcp_hostname:
            description:
            - Configures and specifies value for hostname option over DHCP ip. Note,
              This option shall work only when dhcp is configured as IP.
            - This option is DEPRECATED and is replaced with dhcp which
              accepts dict as input this attribute will be removed after 2023-08-01.
            type: str
          dhcp:
            description: IP Address negotiated via DHCP.
            type: dict
            suboptions:
              enable:
                description: Enable dhcp.
                type: bool
              client_id:
                description: Specify client-id to use.
                type: str
              hostname:
                description: Specify value for hostname option.
                type: str
          pool:
            description: IP Address auto-configured from a local DHCP pool.
            type: str
      ipv6:
        description:
        - IPv6 address to be set for the Layer-3 interface mentioned in I(name) option.
        - The address format is <ipv6 address>/<mask>, the mask is number in range
          0-128 eg. fd5d:12c9:2201:1::1/64
        type: list
        elements: dict
        suboptions:
          address:
            description:
            - Configures the IPv6 address for Interface.
            type: str
          autoconfig:
            description: Obtain address using auto-configuration.
            type: dict
            suboptions:
              enable:
                description: enable auto-configuration.
                type: bool
              default:
                description: Insert default route.
                type: bool
          dhcp:
            description: Obtain a ipv6 address using DHCP.
            type: dict
            suboptions:
              enable:
                description: Enable dhcp.
                type: bool
              rapid_commit:
                description: Enable Rapid-Commit.
                type: bool
          anycast:
            description: Configure as an anycast
            type: bool
          cga:
            description: Use CGA interface identifier
            type: bool
          eui:
            description: Use eui-64 interface identifier
            type: bool
          link_local:
            description: Use link-local address
            type: bool
          segment_routing:
            description: Segment Routing submode
            type: dict
            suboptions:
              enable:
                description: Enable segmented routing.
                type: bool
              default:
                description: Set a command to its defaults.
                type: bool
              ipv6_sr:
                description: Set ipv6_sr.
                type: bool
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS device
        by executing the command B(show running-config | section ^interface).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - rendered
    - gathered
    - parsed
    default: merged
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command
        I(show running-config | section ^interface) executed on device. For state I(parsed) active
        connection to remote host is not required.
    type: str
"""

EXAMPLES = """
# Using state merged

# Before state:
# -------------

# router-ios#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  ip address 10.1.1.1 255.255.255.0
#  duplex auto
#  speed auto
# interface GigabitEthernet0/2
#  description This is test
#  no ip address
#  duplex auto
#  speed 1000
# interface GigabitEthernet0/3
#  description Configured by Ansible Network
#  no ip address
# interface GigabitEthernet0/3.100
#  encapsulation dot1Q 20

- name: Merge provided configuration with device configuration
  cisco.ios.ios_l3_interfaces:
    config:
    - name: GigabitEthernet0/1
      ipv4:
      - address: 192.168.0.1/24
        secondary: true
    - name: GigabitEthernet0/2
      ipv4:
      - address: 192.168.0.2/24
    - name: GigabitEthernet0/3
      ipv6:
      - address: fd5d:12c9:2201:1::1/64
    - name: GigabitEthernet0/3.100
      ipv4:
      - address: 192.168.0.3/24
    state: merged

# Commands Fired:
# ---------------

# "commands": [
#       "interface GigabitEthernet0/1",
#       "ip address 192.168.0.1 255.255.255.0 secondary",
#       "interface GigabitEthernet0/2",
#       "ip address 192.168.0.2 255.255.255.0",
#       "interface GigabitEthernet0/3",
#       "ipv6 address fd5d:12c9:2201:1::1/64",
#       "GigabitEthernet0/3.100",
#       "ip address 192.168.0.3 255.255.255.0",
#     ],

# After state:
# ------------

# router-ios#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  ip address 10.1.1.1 255.255.255.0
#  ip address 192.168.0.1 255.255.255.0 secondary
#  duplex auto
#  speed auto
# interface GigabitEthernet0/2
#  description This is test
#  ip address 192.168.0.2 255.255.255.0
#  duplex auto
#  speed 1000
# interface GigabitEthernet0/3
#  description Configured by Ansible Network
#  ipv6 address FD5D:12C9:2201:1::1/64
# interface GigabitEthernet0/3.100
#  encapsulation dot1Q 20
#  ip address 192.168.0.3 255.255.255.0

# Using state replaced

# Before state:
# -------------

# router-ios#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  ip address 10.1.1.1 255.255.255.0
#  duplex auto
#  speed auto
# interface GigabitEthernet0/2
#  description This is test
#  no ip address
#  duplex auto
#  speed 1000
# interface GigabitEthernet0/3
#  description Configured by Ansible Network
#  ip address 192.168.2.0 255.255.255.0
# interface GigabitEthernet0/3.100
#  encapsulation dot1Q 20
#  ip address 192.168.0.2 255.255.255.0

- name: Replaces device configuration of listed interfaces with provided configuration
  cisco.ios.ios_l3_interfaces:
    config:
    - name: GigabitEthernet0/2
      ipv4:
      - address: 192.168.2.0/24
    - name: GigabitEthernet0/3
      ipv4:
      - dhcp:
          client_id: GigabitEthernet0/2
          hostname: test.com
    - name: GigabitEthernet0/3.100
      ipv4:
      - address: 192.168.0.3/24
        secondary: true
    state: replaced

# Commands Fired:
# ---------------

# "commands": [
#       "interface GigabitEthernet0/1",
#       "ip address 192.168.0.1 255.255.255.0 secondary",
#       "interface GigabitEthernet0/2",
#       "ip address 192.168.0.2 255.255.255.0",
#       "interface GigabitEthernet0/3",
#       "no ip address 192.168.2.0 255.255.255.0",
#       "ip address dhcp client-id GigabitEthernet0/2 hostname test.com",
#       "GigabitEthernet0/3.100",
#       "no ip address 192.168.0.2 255.255.255.0",
#       "ip address 192.168.0.3 255.255.255.0 secondary",
#     ],

# After state:
# ------------

# router-ios#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  ip address 10.1.1.1 255.255.255.0
#  duplex auto
#  speed auto
# interface GigabitEthernet0/2
#  description This is test
#  ip address 192.168.2.1 255.255.255.0
#  duplex auto
#  speed 1000
# interface GigabitEthernet0/3
#  description Configured by Ansible Network
#  ip address dhcp client-id GigabitEthernet0/2 hostname test.com
# interface GigabitEthernet0/3.100
#  encapsulation dot1Q 20
#  ip address 192.168.0.3 255.255.255.0 secondary

# Using state overridden

# Before state:
# -------------

# router-ios#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  ip address 10.1.1.1 255.255.255.0
#  duplex auto
#  speed auto
# interface GigabitEthernet0/2
#  description This is test
#  ip address 192.168.2.1 255.255.255.0
#  duplex auto
#  speed 1000
# interface GigabitEthernet0/3
#  description Configured by Ansible Network
#  ipv6 address FD5D:12C9:2201:1::1/64
# interface GigabitEthernet0/3.100
#  encapsulation dot1Q 20
#  ip address 192.168.0.2 255.255.255.0

- name: Override device configuration of all interfaces with provided configuration
  cisco.ios.ios_l3_interfaces:
    config:
    - name: GigabitEthernet0/2
      ipv4:
      - address: 192.168.0.1/24
    - name: GigabitEthernet0/3.100
      ipv6:
      - autoconfig: true
    state: overridden

# Commands Fired:
# ---------------

# "commands": [
#       "interface GigabitEthernet0/1",
#       "no ip address 10.1.1.1 255.255.255.0",
#       "interface GigabitEthernet0/2",
#       "no ip address 192.168.2.1 255.255.255.0",
#       "ip address 192.168.0.1 255.255.255.0",
#       "interface GigabitEthernet0/3",
#       "no ipv6 address FD5D:12C9:2201:1::1/64",
#       "GigabitEthernet0/3.100",
#       "no ip address 192.168.0.2 255.255.255.0",
#       "ipv6 address autoconfig",
#     ],

# After state:
# ------------

# router-ios#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  no ip address
#  duplex auto
#  speed auto
# interface GigabitEthernet0/2
#  description This is test
#  ip address 192.168.0.1 255.255.255.0
#  duplex auto
#  speed 1000
# interface GigabitEthernet0/3
#  description Configured by Ansible Network
# interface GigabitEthernet0/3.100
#  encapsulation dot1Q 20
#  ipv6 address autoconfig

# Using state Deleted

# Before state:
# -------------

# router-ios#show running-config | section ^interface
# interface GigabitEthernet0/1
#  ip address 192.0.2.10 255.255.255.0
#  shutdown
#  duplex auto
#  speed auto
# interface GigabitEthernet0/2
#  description Configured by Ansible Network
#  ip address 192.168.1.0 255.255.255.0
# interface GigabitEthernet0/3
#  description Configured by Ansible Network
#  ip address 192.168.0.1 255.255.255.0
#  shutdown
#  duplex full
#  speed 10
#  ipv6 address FD5D:12C9:2201:1::1/64
# interface GigabitEthernet0/3.100
#  encapsulation dot1Q 20
#  ip address 192.168.0.2 255.255.255.0

- name: "Delete attributes of given interfaces (NOTE: This won't delete the interfaces itself)"
  cisco.ios.ios_l3_interfaces:
    config:
    - name: GigabitEthernet0/2
    - name: GigabitEthernet0/3.100
    state: deleted

# "commands": [
#       "interface GigabitEthernet0/2",
#       "no ip address 192.168.1.0 255.255.255.0",
#       "GigabitEthernet0/3.100",
#       "no ip address 192.168.0.2 255.255.255.0",
#     ],

# After state:
# -------------

# router-ios#show running-config | section ^interface
# interface GigabitEthernet0/1
#  ip address 192.0.2.10 255.255.255.0
#  shutdown
#  duplex auto
#  speed auto
# interface GigabitEthernet0/2
#  description Configured by Ansible Network
#  no ip address
# interface GigabitEthernet0/3
#  description Configured by Ansible Network
#  ip address 192.168.0.1 255.255.255.0
#  shutdown
#  duplex full
#  speed 10
#  ipv6 address FD5D:12C9:2201:1::1/64
# interface GigabitEthernet0/3.100
#  encapsulation dot1Q 20

# Using state Deleted without any config passed
#"(NOTE: This will delete all of configured L3 resource module attributes from each configured interface)"

# Before state:
# -------------

# router-ios#show running-config | section ^interface
# interface GigabitEthernet0/1
#  ip address 192.0.2.10 255.255.255.0
#  shutdown
#  duplex auto
#  speed auto
# interface GigabitEthernet0/2
#  description Configured by Ansible Network
#  ip address 192.168.1.0 255.255.255.0
# interface GigabitEthernet0/3
#  description Configured by Ansible Network
#  ip address 192.168.0.1 255.255.255.0
#  shutdown
#  duplex full
#  speed 10
#  ipv6 address FD5D:12C9:2201:1::1/64
# interface GigabitEthernet0/3.100
#  encapsulation dot1Q 20
#  ip address 192.168.0.2 255.255.255.0

- name: "Delete L3 attributes of ALL interfaces together (NOTE: This won't delete the interface itself)"
  cisco.ios.ios_l3_interfaces:
    state: deleted

# "commands": [
#       "interface GigabitEthernet0/1",
#       "no ip address 192.0.2.10 255.255.255.0",
#       "interface GigabitEthernet0/2",
#       "no ip address 192.168.1.0 255.255.255.0",
#       "interface GigabitEthernet0/3",
#       "no ip address 192.168.0.1 255.255.255.0",
#       "no ipv6 address FD5D:12C9:2201:1::1/64",
#       "GigabitEthernet0/3.100",
#       "no ip address 192.168.0.2 255.255.255.0",
#     ],

# After state:
# -------------

# router-ios#show running-config | section ^interface
# interface GigabitEthernet0/1
#  no ip address
#  shutdown
#  duplex auto
#  speed auto
# interface GigabitEthernet0/2
#  description Configured by Ansible Network
#  no ip address
# interface GigabitEthernet0/3
#  description Configured by Ansible Network
#  shutdown
#  duplex full
#  speed 10
# interface GigabitEthernet0/3.100
#  encapsulation dot1Q 20

# Using state Gathered

# Before state:
# -------------

# router-ios#sh running-config | section ^interface
# interface GigabitEthernet0/1
#  ip address 203.0.113.27 255.255.255.0
# interface GigabitEthernet0/2
#  ip address 192.0.2.1 255.255.255.0 secondary
#  ip address 192.0.2.2 255.255.255.0
#  ipv6 address 2001:DB8:0:3::/64

- name: Gather listed l3 interfaces with provided configurations
  cisco.ios.ios_l3_interfaces:
    state: gathered

# Module Execution Result:
# ------------------------

# "gathered": [
#         {
#             "ipv4": [
#                 {
#                     "address": "203.0.113.27 255.255.255.0"
#                 }
#             ],
#             "name": "GigabitEthernet0/1"
#         },
#         {
#             "ipv4": [
#                 {
#                     "address": "192.0.2.1 255.255.255.0",
#                     "secondary": true
#                 },
#                 {
#                     "address": "192.0.2.2 255.255.255.0"
#                 }
#             ],
#             "ipv6": [
#                 {
#                     "address": "2001:db8:0:3::/64"
#                 }
#             ],
#             "name": "GigabitEthernet0/2"
#         }
#     ]

# After state:
# ------------

# router-ios#sh running-config | section ^interface
# interface GigabitEthernet0/1
#  ip address 203.0.113.27 255.255.255.0
# interface GigabitEthernet0/2
#  ip address 192.0.2.1 255.255.255.0 secondary
#  ip address 192.0.2.2 255.255.255.0
#  ipv6 address 2001:DB8:0:3::/64

# Using state Rendered

- name: Render the commands for provided configuration
  cisco.ios.ios_l3_interfaces:
    config:
    - name: GigabitEthernet0/1
      ipv4:
      - dhcp:
          client_id: GigabitEthernet0/0
          hostname: test.com
    - name: GigabitEthernet0/2
      ipv4:
      - address: 198.51.100.1/24
        secondary: true
      - address: 198.51.100.2/24
      ipv6:
      - address: 2001:db8:0:3::/64
    state: rendered

# Module Execution Result:
# ------------------------

# "rendered": [
#         "interface GigabitEthernet0/1",
#         "ip address dhcp client-id GigabitEthernet 0/0 hostname test.com",
#         "interface GigabitEthernet0/2",
#         "ip address 198.51.100.1 255.255.255.0 secondary",
#         "ip address 198.51.100.2 255.255.255.0",
#         "ipv6 address 2001:db8:0:3::/64"
#     ]

# Using state Parsed

# File: parsed.cfg
# ----------------
#
# interface GigabitEthernet0/1
#  ip address dhcp client-id GigabitEthernet 0/0 hostname test.com
# interface GigabitEthernet0/2
#  ip address 198.51.100.1 255.255.255.0
#  ip address 198.51.100.2 255.255.255.0 secondary
#  ipv6 address 2001:db8:0:3::/64

- name: Parse the commands for provided configuration
  cisco.ios.ios_l3_interfaces:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Module Execution Result:
# ------------------------

# "parsed": [
#         {
#             "ipv4": [
#                 {
#                     "dhcp": {
#                         "client_id": GigabitEthernet0/0,
#                         "hostname": "test.com"
#                     }
#                 }
#             ],
#             "name": "GigabitEthernet0/1"
#         },
#         {
#             "ipv4": [
#                 {
#                     "address": "198.51.100.1/24",
#                     "secondary": true
#                 },
#                 {
#                     "address": "198.51.100.2/24"
#                 }
#             ],
#             "ipv6": [
#                 {
#                     "address": "2001:db8:0:3::/64"
#                 }
#             ],
#             "name": "GigabitEthernet0/2"
#         }
#     ]

"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when state is I(merged), I(replaced), I(overridden), I(deleted) or I(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when state is I(merged), I(replaced), I(overridden), I(deleted) or I(purged)
  type: list
  sample:
    - "ip address 192.168.0.3 255.255.255.0"
    - "ipv6 address dhcp rapid-commit"
    - "ipv6 address fd5d:12c9:2201:1::1/64 anycast"
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when state is I(rendered)
  type: list
  sample:
    - "ipv6 address FD5D:12C9:2201:1::1/64"
    - "ip address 192.168.0.3 255.255.255.0"
    - "ip address autoconfig"
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when state is I(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when state is I(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.l3_interfaces.l3_interfaces import (
    L3_interfacesArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.l3_interfaces.l3_interfaces import (
    L3_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=L3_interfacesArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = L3_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2019, Dag Wieers (@dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: mso_schema_site_anp_epg_staticleaf
short_description: Manage site-local EPG static leafs in schema template
description:
- Manage site-local EPG static leafs in schema template on Cisco ACI Multi-Site.
author:
- Dag Wieers (@dagwieers)
options:
  schema:
    description:
    - The name of the schema.
    type: str
    required: yes
  site:
    description:
    - The name of the site.
    type: str
    required: yes
  template:
    description:
    - The name of the template.
    type: str
    required: yes
  anp:
    description:
    - The name of the ANP.
    type: str
    required: yes
  epg:
    description:
    - The name of the EPG.
    type: str
    required: yes
  pod:
    description:
    - The pod of the static leaf.
    type: str
  leaf:
    description:
    - The path of the static leaf.
    type: str
    aliases: [ name ]
  vlan:
    description:
    - The VLAN id of the static leaf.
    type: int
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
notes:
- The ACI MultiSite PATCH API has a deficiency requiring some objects to be referenced by index.
  This can cause silent corruption on concurrent access when changing/removing on object as
  the wrong object may be referenced. This module is affected by this deficiency.
seealso:
- module: cisco.mso.mso_schema_site_anp_epg
- module: cisco.mso.mso_schema_template_anp_epg
extends_documentation_fragment: cisco.mso.modules
'''

EXAMPLES = r'''
- name: Add a new static leaf to a site EPG
  cisco.mso.mso_schema_site_anp_epg_staticleaf:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    leaf: Leaf1
    vlan: 123
    state: present
  delegate_to: localhost

- name: Remove a static leaf from a site EPG
  cisco.mso.mso_schema_site_anp_epg_staticleaf:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    leaf: Leaf1
    state: absent
  delegate_to: localhost

- name: Query a specific site EPG static leaf
  cisco.mso.mso_schema_site_anp_epg_staticleaf:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    leaf: Leaf1
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all site EPG static leafs
  cisco.mso.mso_schema_site_anp_epg_staticleaf:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    state: query
  delegate_to: localhost
  register: query_result
'''

RETURN = r'''
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type='str', required=True),
        site=dict(type='str', required=True),
        template=dict(type='str', required=True),
        anp=dict(type='str', required=True),
        epg=dict(type='str', required=True),
        pod=dict(type='str'),  # This parameter is not required for querying all objects
        leaf=dict(type='str', aliases=['name']),
        vlan=dict(type='int'),
        state=dict(type='str', default='present', choices=['absent', 'present', 'query']),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ['state', 'absent', ['pod', 'leaf', 'vlan']],
            ['state', 'present', ['pod', 'leaf', 'vlan']],
        ],
    )

    schema = module.params.get('schema')
    site = module.params.get('site')
    template = module.params.get('template').replace(' ', '')
    anp = module.params.get('anp')
    epg = module.params.get('epg')
    pod = module.params.get('pod')
    leaf = module.params.get('leaf')
    vlan = module.params.get('vlan')
    state = module.params.get('state')

    leafpath = 'topology/{0}/node-{1}'.format(pod, leaf)

    mso = MSOModule(module)

    # Get schema objects
    schema_id, schema_path, schema_obj = mso.query_schema(schema)

    # Get site
    site_id = mso.lookup_site(site)

    # Get site_idx
    if 'sites' not in schema_obj:
        mso.fail_json(msg="No site associated with template '{0}'. Associate the site with the template using mso_schema_site.".format(template))
    sites = [(s.get('siteId'), s.get('templateName')) for s in schema_obj.get('sites')]
    if (site_id, template) not in sites:
        mso.fail_json(msg="Provided site/template '{0}-{1}' does not exist. Existing sites/templates: {2}".format(site, template, ', '.join(sites)))

    # Schema-access uses indexes
    site_idx = sites.index((site_id, template))
    # Path-based access uses site_id-template
    site_template = '{0}-{1}'.format(site_id, template)

    # Get ANP
    anp_ref = mso.anp_ref(schema_id=schema_id, template=template, anp=anp)
    anps = [a.get('anpRef') for a in schema_obj.get('sites')[site_idx]['anps']]
    if anp_ref not in anps:
        mso.fail_json(msg="Provided anp '{0}' does not exist. Existing anps: {1}".format(anp, ', '.join(anps)))
    anp_idx = anps.index(anp_ref)

    # Get EPG
    epg_ref = mso.epg_ref(schema_id=schema_id, template=template, anp=anp, epg=epg)
    epgs = [e.get('epgRef') for e in schema_obj.get('sites')[site_idx]['anps'][anp_idx]['epgs']]
    if epg_ref not in epgs:
        mso.fail_json(msg="Provided epg '{0}' does not exist. Existing epgs: {1}".format(epg, ', '.join(epgs)))
    epg_idx = epgs.index(epg_ref)

    # Get Leaf
    leafs = [(leaf.get('path'), leaf.get('portEncapVlan')) for leaf in schema_obj.get('sites')[site_idx]['anps'][anp_idx]['epgs'][epg_idx]['staticLeafs']]
    if (leafpath, vlan) in leafs:
        leaf_idx = leafs.index((leafpath, vlan))
        # FIXME: Changes based on index are DANGEROUS
        leaf_path = '/sites/{0}/anps/{1}/epgs/{2}/staticLeafs/{3}'.format(site_template, anp, epg, leaf_idx)
        mso.existing = schema_obj.get('sites')[site_idx]['anps'][anp_idx]['epgs'][epg_idx]['staticLeafs'][leaf_idx]

    if state == 'query':
        if leaf is None or vlan is None:
            mso.existing = schema_obj.get('sites')[site_idx]['anps'][anp_idx]['epgs'][epg_idx]['staticLeafs']
        elif not mso.existing:
            mso.fail_json(msg="Static leaf '{leaf}/{vlan}' not found".format(leaf=leaf, vlan=vlan))
        mso.exit_json()

    leafs_path = '/sites/{0}/anps/{1}/epgs/{2}/staticLeafs'.format(site_template, anp, epg)
    ops = []

    mso.previous = mso.existing
    if state == 'absent':
        if mso.existing:
            mso.sent = mso.existing = {}
            ops.append(dict(op='remove', path=leaf_path))

    elif state == 'present':
        payload = dict(
            path=leafpath,
            portEncapVlan=vlan,
        )

        mso.sanitize(payload, collate=True)

        if mso.existing:
            ops.append(dict(op='replace', path=leaf_path, value=mso.sent))
        else:
            ops.append(dict(op='add', path=leafs_path + '/-', value=mso.sent))

        mso.existing = mso.proposed

    if not module.check_mode:
        mso.request(schema_path, method='PATCH', data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()



# (c) 2021 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible_collections.cisco.iosxr.tests.unit.compat.mock import patch
from ansible_collections.cisco.iosxr.plugins.modules import (
    iosxr_lag_interfaces,
)
from ansible_collections.cisco.iosxr.tests.unit.modules.utils import (
    set_module_args,
)
from .iosxr_module import TestIosxrModule, load_fixture


class TestIosxrLagInterfacesModule(TestIosxrModule):
    module = iosxr_lag_interfaces

    def setUp(self):
        super(TestIosxrLagInterfacesModule, self).setUp()

        self.mock_get_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.get_config"
        )
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.load_config"
        )
        self.load_config = self.mock_load_config.start()

        self.mock_get_resource_connection_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base.get_resource_connection"
        )
        self.get_resource_connection_config = (
            self.mock_get_resource_connection_config.start()
        )

        self.mock_get_resource_connection_facts = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.facts.facts.get_resource_connection"
        )
        self.get_resource_connection_facts = (
            self.mock_get_resource_connection_facts.start()
        )
        self.mock_execute_show_command = patch(
            "ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.lag_interfaces.lag_interfaces.Lag_interfacesFacts.get_config"
        )
        self.execute_show_command = self.mock_execute_show_command.start()

    def tearDown(self):
        super(TestIosxrLagInterfacesModule, self).tearDown()
        self.mock_get_resource_connection_config.stop()
        self.mock_get_resource_connection_facts.stop()
        self.mock_get_config.stop()
        self.mock_load_config.stop()
        self.mock_execute_show_command.stop()

    def _prepare(self):
        def load_from_file(*args, **kwargs):
            return load_fixture("iosxr_lag_interface_config.cfg")

        self.execute_show_command.side_effect = load_from_file

    def test_iosxr_lag_interfaces_merged_idempotent(self):
        self._prepare()
        set_module_args(
            dict(
                config=[
                    dict(
                        name="Bundle-Ether10",
                        mode="active",
                        members=[
                            dict(
                                member="GigabitEthernet0/0/0/0", mode="inherit"
                            ),
                            dict(
                                member="GigabitEthernet0/0/0/1", mode="passive"
                            ),
                        ],
                        links=dict(max_active=10, min_active=2),
                    ),
                    dict(
                        name="Bundle-Ether11",
                        mode="active",
                        members=[
                            dict(
                                member="GigabitEthernet0/0/0/8", mode="passive"
                            ),
                            dict(
                                member="GigabitEthernet0/0/0/9", mode="passive"
                            ),
                        ],
                    ),
                ],
                state="merged",
            )
        )
        self.execute_module(changed=False, commands=[])

    def test_iosxr_lag_interfaces_merged(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        name="Bundle-Ether10",
                        mode="active",
                        members=[
                            dict(
                                member="GigabitEthernet0/0/0/0", mode="inherit"
                            ),
                            dict(
                                member="GigabitEthernet0/0/0/1", mode="passive"
                            ),
                        ],
                        links=dict(max_active=10, min_active=2),
                    ),
                    dict(
                        name="Bundle-Ether11",
                        mode="active",
                        members=[
                            dict(
                                member="GigabitEthernet0/0/0/8", mode="passive"
                            ),
                            dict(
                                member="GigabitEthernet0/0/0/9", mode="passive"
                            ),
                        ],
                    ),
                ],
                state="merged",
            )
        )
        commands = [
            "interface Bundle-Ether10",
            "bundle minimum-active links 2",
            "bundle maximum-active links 10",
            "lacp mode active",
            "interface GigabitEthernet0/0/0/1",
            "bundle id 10 mode passive",
            "interface GigabitEthernet0/0/0/0",
            "bundle id 10 mode inherit",
            "interface Bundle-Ether11",
            "lacp mode active",
            "interface GigabitEthernet0/0/0/8",
            "bundle id 11 mode passive",
            "interface GigabitEthernet0/0/0/9",
            "bundle id 11 mode passive",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_iosxr_lag_interfaces_replaced(self):
        self._prepare()
        set_module_args(
            dict(
                config=[
                    dict(
                        name="Bundle-Ether10",
                        mode="passive",
                        members=[
                            dict(
                                member="GigabitEthernet0/0/0/0", mode="passive"
                            )
                        ],
                    ),
                    dict(name="Bundle-Ether12", mode="active"),
                ],
                state="replaced",
            )
        )
        commands = [
            "interface Bundle-Ether10",
            "no bundle maximum-active links 10",
            "no bundle minimum-active links 2",
            "lacp mode passive",
            "interface GigabitEthernet0/0/0/1",
            "no bundle id",
            "interface GigabitEthernet0/0/0/0",
            "bundle id 10 mode passive",
            "interface Bundle-Ether12",
            "lacp mode active",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_iosxr_lag_interfaces_deleted(self):
        self._prepare()
        set_module_args(dict(state="deleted"))

        commands = [
            "interface Bundle-Ether10",
            "no bundle maximum-active links 10",
            "no bundle minimum-active links 2",
            "no lacp mode active",
            "interface GigabitEthernet0/0/0/0",
            "no bundle id",
            "interface GigabitEthernet0/0/0/1",
            "no bundle id",
            "interface Bundle-Ether11",
            "no lacp mode active",
            "interface GigabitEthernet0/0/0/8",
            "no bundle id",
            "interface GigabitEthernet0/0/0/9",
            "no bundle id",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_iosxr_lag_interfaces_rendered(self):
        set_module_args(
            dict(
                config=[
                    dict(
                        name="Bundle-Ether10",
                        mode="active",
                        members=[
                            dict(
                                member="GigabitEthernet0/0/0/0", mode="passive"
                            ),
                            dict(
                                member="GigabitEthernet0/0/0/1", mode="passive"
                            ),
                        ],
                        links=dict(max_active=10, min_active=2),
                    ),
                    dict(
                        name="Bundle-Ether11",
                        mode="active",
                        members=[
                            dict(
                                member="GigabitEthernet0/0/0/8", mode="passive"
                            ),
                            dict(
                                member="GigabitEthernet0/0/0/9", mode="passive"
                            ),
                        ],
                    ),
                ],
                state="rendered",
            )
        )

        commands = [
            "interface Bundle-Ether10",
            "bundle minimum-active links 2",
            "bundle maximum-active links 10",
            "lacp mode active",
            "interface GigabitEthernet0/0/0/1",
            "bundle id 10 mode passive",
            "interface GigabitEthernet0/0/0/0",
            "bundle id 10 mode passive",
            "interface Bundle-Ether11",
            "lacp mode active",
            "interface GigabitEthernet0/0/0/8",
            "bundle id 11 mode passive",
            "interface GigabitEthernet0/0/0/9",
            "bundle id 11 mode passive",
        ]
        result = self.execute_module(changed=False)
        self.assertEqual(sorted(result["rendered"]), sorted(commands))

    def test_iosxr_lag_interfaces_parsed(self):
        self.maxDiff = None
        set_module_args(
            dict(
                running_config="interface Bundle-Ether10\r\n lacp mode active\r\n bundle maximum-active "
                "links 10\r\n bundle minimum-active links 2\r\n!\r\ninterface Bundle-Ether11"
                "\r\nlacp mode active\r\n!\r\ninterface GigabitEthernet0/0/0/0\r\n description "
                '"GigabitEthernet - 0"\r\n bundle id 10 mode inherit\r\n!\r\ninterface '
                "GigabitEthernet0/0/0/1"
                '\r\n description "GigabitEthernet - 2"\r\n  bundle id 10 mode passive\r\n!\r\n'
                'interface GigabitEthernet0/0/0/8\r\n description "GigabitEthernet - 8"'
                "\r\n bundle id 11 mode passive"
                "\r\n!\r\ninterface GigabitEthernet0/0/0/9\r\n description "
                '"GigabitEthernet - 9"\r\n bundle id 11 mode passive\r\n!',
                state="parsed",
            )
        )
        result = self.execute_module(changed=False)
        print(result["parsed"])
        parsed_list = [
            {
                "links": {"max_active": 10, "min_active": 2},
                "members": [
                    {"member": "GigabitEthernet0/0/0/0", "mode": "inherit"},
                    {"member": "GigabitEthernet0/0/0/1", "mode": "passive"},
                ],
                "mode": "active",
                "name": "Bundle-Ether10",
            },
            {
                "members": [
                    {"member": "GigabitEthernet0/0/0/8", "mode": "passive"},
                    {"member": "GigabitEthernet0/0/0/9", "mode": "passive"},
                ],
                "mode": "active",
                "name": "Bundle-Ether11",
            },
        ]
        self.assertEqual(parsed_list, result["parsed"])

    def test_iosxr_lag_interfaces_overridden(self):
        self.maxDiff = None
        self._prepare()
        set_module_args(
            dict(
                config=[
                    dict(
                        name="Bundle-Ether11",
                        mode="active",
                        members=[
                            dict(
                                member="GigabitEthernet0/0/0/0", mode="active"
                            ),
                            dict(
                                member="GigabitEthernet0/0/0/1", mode="active"
                            ),
                        ],
                        links=dict(max_active=10, min_active=5),
                    )
                ],
                state="overridden",
            )
        )
        commands = [
            "interface Bundle-Ether10",
            "no bundle maximum-active links 10",
            "no bundle minimum-active links 2",
            "no lacp mode active",
            "interface GigabitEthernet0/0/0/0",
            "no bundle id",
            "interface GigabitEthernet0/0/0/1",
            "no bundle id",
            "interface Bundle-Ether11",
            "bundle minimum-active links 5",
            "bundle maximum-active links 10",
            "interface GigabitEthernet0/0/0/8",
            "no bundle id",
            "interface GigabitEthernet0/0/0/9",
            "no bundle id",
            "interface GigabitEthernet0/0/0/0",
            "bundle id 11 mode active",
            "interface GigabitEthernet0/0/0/1",
            "bundle id 11 mode active",
        ]

        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))



# (c) 2019 Telstra Corporation Limited
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from pprint import pprint

from ansible_collections.community.aws.plugins.modules.aws_acm import chain_compare
from ansible_collections.community.aws.plugins.modules.aws_acm import pem_chain_split
from ansible.module_utils._text import to_text


def test_chain_compare():

    # The functions we're testing take module as an argument
    # Just so they can call module.fail_json
    # Let's just use None for the unit tests,
    # Because they shouldn't fail
    # And if they do, fail_json is not applicable
    module = None

    fixture_suffix = 'tests/unit/plugins/modules/fixtures/certs'

    # Test chain split function on super simple (invalid) certs
    expected = ['aaa', 'bbb', 'ccc']

    for fname in ['simple-chain-a.cert', 'simple-chain-b.cert']:
        path = fixture_suffix + '/' + fname
        with open(path, 'r') as f:
            pem = to_text(f.read())
        actual = pem_chain_split(module, pem)
        actual = [a.strip() for a in actual]
        if actual != expected:
            print("Expected:")
            pprint(expected)
            print("Actual:")
            pprint(actual)
            raise AssertionError("Failed to properly split %s" % fname)

    # Now test real chains
    # chains with same same_as should be considered equal
    test_chains = [
        {  # Original Cert chain
            'path': fixture_suffix + '/chain-1.0.cert',
            'same_as': 1,
            'length': 3
        },
        {  # Same as 1.0, but longer PEM lines
            'path': fixture_suffix + '/chain-1.1.cert',
            'same_as': 1,
            'length': 3
        },
        {  # Same as 1.0, but without the stuff before each --------
            'path': fixture_suffix + '/chain-1.2.cert',
            'same_as': 1,
            'length': 3
        },
        {  # Same as 1.0, but in a different order, so should be considered different
            'path': fixture_suffix + '/chain-1.3.cert',
            'same_as': 2,
            'length': 3
        },
        {  # Same as 1.0, but with last link missing
            'path': fixture_suffix + '/chain-1.4.cert',
            'same_as': 3,
            'length': 2
        },
        {  # Completely different cert chain to all the others
            'path': fixture_suffix + '/chain-4.cert',
            'same_as': 4,
            'length': 3
        },
        {  # Single cert
            'path': fixture_suffix + '/a.pem',
            'same_as': 5,
            'length': 1
        },
        {  # a different, single cert
            'path': fixture_suffix + '/b.pem',
            'same_as': 6,
            'length': 1
        }
    ]

    for chain in test_chains:
        with open(chain['path'], 'r') as f:
            chain['pem_text'] = to_text(f.read())

        # Test to make sure our regex isn't too greedy
        chain['split'] = pem_chain_split(module, chain['pem_text'])
        if len(chain['split']) != chain['length']:
            print("Cert before split")
            print(chain['pem_text'])
            print("Cert after split")
            pprint(chain['split'])
            print("path: %s" % chain['path'])
            print("Expected chain length: %d" % chain['length'])
            print("Actual chain length: %d" % len(chain['split']))
            raise AssertionError("Chain %s was not split properly" % chain['path'])

    for chain_a in test_chains:
        for chain_b in test_chains:
            expected = (chain_a['same_as'] == chain_b['same_as'])

            # Now test the comparison function
            actual = chain_compare(module, chain_a['pem_text'], chain_b['pem_text'])
            if expected != actual:
                print("Error, unexpected comparison result between \n%s\nand\n%s" % (chain_a['path'], chain_b['path']))
                print("Expected %s got %s" % (str(expected), str(actual)))
            assert(expected == actual)



#
# (c) 2021, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from textwrap import dedent
from ansible_collections.cisco.ios.tests.unit.compat.mock import patch
from ansible_collections.cisco.ios.plugins.modules import ios_ntp_global
from ansible_collections.cisco.ios.tests.unit.modules.utils import (
    set_module_args,
)
from .ios_module import TestIosModule


class TestIosNtpGlobalModule(TestIosModule):
    module = ios_ntp_global

    def setUp(self):
        super(TestIosNtpGlobalModule, self).setUp()

        self.mock_get_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.get_config"
        )
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network.Config.load_config"
        )
        self.load_config = self.mock_load_config.start()

        self.mock_get_resource_connection_config = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base."
            "get_resource_connection"
        )
        self.get_resource_connection_config = (
            self.mock_get_resource_connection_config.start()
        )

        self.mock_get_resource_connection_facts = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module_base."
            "get_resource_connection"
        )
        self.get_resource_connection_facts = (
            self.mock_get_resource_connection_facts.start()
        )

        self.mock_edit_config = patch(
            "ansible_collections.cisco.ios.plugins.module_utils.network.ios.providers.providers.CliProvider.edit_config"
        )
        self.edit_config = self.mock_edit_config.start()

        self.mock_execute_show_command = patch(
            "ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.ntp_global.ntp_global."
            "Ntp_globalFacts.get_ntp_data"
        )
        self.execute_show_command = self.mock_execute_show_command.start()

    def tearDown(self):
        super(TestIosNtpGlobalModule, self).tearDown()
        self.mock_get_resource_connection_config.stop()
        self.mock_get_resource_connection_facts.stop()
        self.mock_edit_config.stop()
        self.mock_get_config.stop()
        self.mock_load_config.stop()
        self.mock_execute_show_command.stop()

    def test_ios_ntp_global_merged_idempotent(self):
        self.execute_show_command.return_value = dedent(
            """\
            ntp allow mode control 4
            ntp allow mode private
            ntp authenticate
            ntp broadcastdelay 22
            ntp clock-period 5
            ntp logging
            ntp master 4
            ntp max-associations 34
            ntp maxdistance 3
            ntp mindistance 10
            ntp orphan 4
            ntp panic update
            ntp source GigabitEthernet0/1
            ntp update-calendar
            ntp access-group ipv4 peer DHCP-Server kod
            ntp access-group ipv6 peer preauth_ipv6_acl kod
            ntp access-group peer 2 kod
            ntp access-group query-only 10
            ntp authentication-key 2 md5 SomeSecurePassword 7
            ntp peer 172.16.1.10 version 2
            ntp peer 172.16.1.11 key 2 minpoll 5 prefer version 2
            ntp peer ip checkPeerDomainIpv4.com prefer
            ntp peer ipv6 checkPeerDomainIpv6.com
            ntp peer ipv6 testPeerDomainIpv6.com prefer
            ntp server 172.16.1.12 version 2
            ntp server ipv6 checkServerDomainIpv6.com
            ntp server 172.16.1.13 source GigabitEthernet0/1
            ntp trusted-key 3 - 13
            ntp trusted-key 21
            """
        )
        set_module_args(
            dict(
                config=dict(
                    access_group=dict(
                        peer=[
                            dict(access_list="2", kod=True),
                            dict(
                                access_list="preauth_ipv6_acl",
                                ipv6=True,
                                kod=True,
                            ),
                        ]
                    ),
                    allow=dict(control=dict(rate_limit=4)),
                    authenticate=True,
                    authentication_keys=[
                        dict(
                            algorithm="md5",
                            encryption=7,
                            id=2,
                            key="SomeSecurePassword",
                        )
                    ],
                    broadcast_delay=22,
                    logging=True,
                    master=dict(stratum=4),
                    max_associations=34,
                    max_distance=3,
                    min_distance=10,
                    orphan=4,
                    panic_update=True,
                    peers=[
                        dict(peer="172.16.1.10", version=2),
                        dict(
                            key_id=2,
                            minpoll=5,
                            peer="172.16.1.11",
                            prefer=True,
                            version=2,
                        ),
                        dict(
                            peer="checkPeerDomainIpv4.com",
                            prefer=True,
                            use_ipv4=True,
                        ),
                        dict(peer="checkPeerDomainIpv6.com", use_ipv6=True),
                        dict(
                            peer="testPeerDomainIpv6.com",
                            prefer=True,
                            use_ipv6=True,
                        ),
                    ],
                    servers=[
                        dict(server="172.16.1.12", version=2),
                        dict(
                            server="172.16.1.13", source="GigabitEthernet0/1"
                        ),
                        dict(
                            server="checkServerDomainIpv6.com", use_ipv6=True
                        ),
                    ],
                    source="GigabitEthernet0/1",
                    trusted_keys=[
                        dict(range_start=21),
                        dict(range_end=13, range_start=3),
                    ],
                    update_calendar=True,
                ),
                state="merged",
            )
        )
        commands = []
        result = self.execute_module(changed=False)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_ios_ntp_global_merged(self):
        self.execute_show_command.return_value = dedent(
            """\
            ntp allow mode control 4
            ntp allow mode private
            """
        )
        set_module_args(
            dict(
                config=dict(
                    access_group=dict(
                        peer=[
                            dict(access_list="2", kod=True),
                            dict(
                                access_list="preauth_ipv6_acl",
                                ipv6=True,
                                kod=True,
                            ),
                        ]
                    ),
                    allow=dict(control=dict(rate_limit=4)),
                    authenticate=True,
                    authentication_keys=[
                        dict(
                            algorithm="md5",
                            encryption=7,
                            id=2,
                            key="SomeSecurePassword",
                        )
                    ],
                    broadcast_delay=22,
                    logging=True,
                    master=dict(stratum=4),
                    max_associations=34,
                    max_distance=3,
                    min_distance=10,
                    orphan=4,
                    panic_update=True,
                    peers=[
                        dict(peer="172.16.1.10", version=2),
                        dict(
                            key=2,
                            minpoll=5,
                            peer="172.16.1.11",
                            prefer=True,
                            version=2,
                        ),
                        dict(
                            peer="checkPeerDomainIpv4.com",
                            prefer=True,
                            use_ipv4=True,
                        ),
                        dict(peer="checkPeerDomainIpv6.com", use_ipv6=True),
                        dict(
                            peer="testPeerDomainIpv6.com",
                            prefer=True,
                            use_ipv6=True,
                        ),
                    ],
                    servers=[
                        dict(server="172.16.1.12", version=2),
                        dict(server="172.16.1.110", key_id=2),
                        dict(
                            server="172.16.1.13", source="GigabitEthernet0/1"
                        ),
                        dict(
                            server="checkServerDomainIpv6.com", use_ipv6=True
                        ),
                    ],
                    source="GigabitEthernet0/1",
                    trusted_keys=[
                        dict(range_start=21),
                        dict(range_end=13, range_start=3),
                    ],
                    update_calendar=True,
                ),
                state="merged",
            )
        )
        commands = [
            "ntp authenticate",
            "ntp broadcastdelay 22",
            "ntp logging",
            "ntp master 4",
            "ntp max-associations 34",
            "ntp maxdistance 3",
            "ntp mindistance 10",
            "ntp orphan 4",
            "ntp panic update",
            "ntp source GigabitEthernet0/1",
            "ntp update-calendar",
            "ntp access-group peer 2 kod",
            "ntp access-group ipv6 peer preauth_ipv6_acl kod",
            "ntp authentication-key 2 md5 SomeSecurePassword 7",
            "ntp peer 172.16.1.10 version 2",
            "ntp peer 172.16.1.11 key 2 minpoll 5 prefer version 2",
            "ntp peer ip checkPeerDomainIpv4.com prefer",
            "ntp peer ipv6 checkPeerDomainIpv6.com",
            "ntp peer ipv6 testPeerDomainIpv6.com prefer",
            "ntp server 172.16.1.12 version 2",
            "ntp server 172.16.1.110 key 2",
            "ntp server 172.16.1.13 source GigabitEthernet0/1",
            "ntp server ipv6 checkServerDomainIpv6.com",
            "ntp trusted-key 21",
            "ntp trusted-key 3 - 13",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_ios_ntp_global_deleted(self):
        self.execute_show_command.return_value = dedent(
            """\
            ntp allow mode control 4
            ntp allow mode private
            ntp authenticate
            ntp broadcastdelay 22
            ntp clock-period 5
            ntp logging
            ntp master 4
            ntp max-associations 34
            ntp maxdistance 3
            ntp mindistance 10
            ntp orphan 4
            ntp panic update
            ntp source GigabitEthernet0/1
            ntp update-calendar
            ntp access-group ipv4 peer DHCP-Server kod
            ntp access-group ipv6 peer preauth_ipv6_acl kod
            ntp access-group peer 2 kod
            ntp access-group query-only 10
            ntp authentication-key 2 md5 SomeSecurePassword 7
            ntp peer 172.16.1.10 version 2
            ntp peer 172.16.1.11 key 2 minpoll 5 prefer version 2
            ntp peer ip checkPeerDomainIpv4.com prefer
            ntp peer ipv6 checkPeerDomainIpv6.com
            ntp peer ipv6 testPeerDomainIpv6.com prefer
            ntp server 172.16.1.12 version 2
            ntp server ipv6 checkServerDomainIpv6.com
            ntp server 172.16.1.13 source GigabitEthernet0/1
            ntp trusted-key 3 - 13
            ntp trusted-key 21
            """
        )
        set_module_args(dict(config=dict(), state="deleted"))
        commands = [
            "no ntp allow mode control 4",
            "no ntp allow mode private",
            "no ntp authenticate",
            "no ntp broadcastdelay 22",
            "no ntp clock-period 5",
            "no ntp logging",
            "no ntp master 4",
            "no ntp max-associations 34",
            "no ntp maxdistance 3",
            "no ntp mindistance 10",
            "no ntp orphan 4",
            "no ntp panic update",
            "no ntp source GigabitEthernet0/1",
            "no ntp update-calendar",
            "no ntp access-group peer 2 kod",
            "no ntp access-group ipv4 peer DHCP-Server kod",
            "no ntp access-group ipv6 peer preauth_ipv6_acl kod",
            "no ntp access-group query-only 10",
            "no ntp authentication-key 2 md5 SomeSecurePassword 7",
            "no ntp peer 172.16.1.10 version 2",
            "no ntp peer 172.16.1.11 key 2 minpoll 5 prefer version 2",
            "no ntp peer ip checkPeerDomainIpv4.com prefer",
            "no ntp peer ipv6 checkPeerDomainIpv6.com",
            "no ntp peer ipv6 testPeerDomainIpv6.com prefer",
            "no ntp server 172.16.1.12 version 2",
            "no ntp server 172.16.1.13 source GigabitEthernet0/1",
            "no ntp server ipv6 checkServerDomainIpv6.com",
            "no ntp trusted-key 21",
            "no ntp trusted-key 3 - 13",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_ios_ntp_global_deleted_blank(self):
        self.execute_show_command.return_value = dedent(
            """\
            """
        )
        set_module_args(dict(config=dict(), state="deleted"))
        commands = []
        result = self.execute_module(changed=False)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_ios_ntp_global_replaced_overridden(self):
        """both the replaced and overridden states are supported to have same behaviour"""
        self.execute_show_command.return_value = dedent(
            """\
            ntp allow mode control 4
            ntp allow mode private
            ntp authenticate
            ntp broadcastdelay 22
            ntp clock-period 15
            ntp logging
            ntp master 14
            ntp max-associations 134
            ntp maxdistance 3
            ntp mindistance 100
            ntp orphan 4
            ntp panic update
            ntp source Loopback888
            ntp update-calendar
            ntp access-group ipv4 peer DHCPAC kod
            ntp access-group ipv6 peer preauth_ipv6_acl kod
            ntp access-group peer 2 kod
            ntp access-group query-only 10
            ntp authentication-key 2 md5 SomeSecurePassword 7
            ntp peer 172.16.1.9 version 2
            ntp peer 172.16.1.1 key 2 minpoll 5 prefer version 2
            ntp peer ip checkPeerDomainIpv4.com prefer
            ntp peer ipv6 checkPeerDomainIpv6.com
            ntp peer ipv6 testPeerDomainIpv6.com prefer
            ntp server 172.16.1.19 version 2
            ntp server ipv6 checkServerDomainIpv6.com
            ntp server 172.16.1.111 source GigabitEthernet0/1
            ntp trusted-key 3 - 130
            ntp trusted-key 21
            """
        )
        set_module_args(
            dict(
                config=dict(
                    access_group=dict(
                        peer=[
                            dict(access_list="2", kod=True),
                            dict(
                                access_list="preauth_ipv6_acl",
                                ipv6=True,
                                kod=True,
                            ),
                        ]
                    ),
                    allow=dict(control=dict(rate_limit=4)),
                    authenticate=True,
                    authentication_keys=[
                        dict(
                            algorithm="md5",
                            encryption=7,
                            id=2,
                            key="SomeSecurePassword",
                        )
                    ],
                    broadcast_delay=22,
                    logging=True,
                    master=dict(stratum=4),
                    max_associations=34,
                    max_distance=3,
                    min_distance=10,
                    orphan=4,
                    panic_update=True,
                    peers=[
                        dict(peer="172.16.1.10", version=2),
                        dict(
                            key=2,
                            minpoll=5,
                            peer="172.16.1.11",
                            prefer=True,
                            version=2,
                        ),
                        dict(
                            peer="checkPeerDomainIpv4.com",
                            prefer=True,
                            use_ipv4=True,
                        ),
                        dict(peer="checkPeerDomainIpv6.com", use_ipv6=True),
                        dict(
                            peer="testPeerDomainIpv6.com",
                            prefer=True,
                            use_ipv6=True,
                        ),
                    ],
                    servers=[
                        dict(server="172.16.1.12", version=2),
                        dict(
                            server="172.16.1.13", source="GigabitEthernet0/1"
                        ),
                        dict(
                            server="checkServerDomainIpv6.com", use_ipv6=True
                        ),
                    ],
                    source="GigabitEthernet0/1",
                    trusted_keys=[
                        dict(range_start=21),
                        dict(range_end=13, range_start=3),
                    ],
                    update_calendar=True,
                ),
                state="replaced",
            )
        )
        commands = [
            "no ntp allow mode private",
            "no ntp clock-period 15",
            "ntp master 4",
            "ntp max-associations 34",
            "ntp mindistance 10",
            "ntp source GigabitEthernet0/1",
            "no ntp access-group ipv4 peer DHCPAC kod",
            "no ntp access-group query-only 10",
            "ntp peer 172.16.1.10 version 2",
            "ntp peer 172.16.1.11 key 2 minpoll 5 prefer version 2",
            "no ntp peer 172.16.1.1 key 2 minpoll 5 prefer version 2",
            "no ntp peer 172.16.1.9 version 2",
            "ntp server 172.16.1.12 version 2",
            "ntp server 172.16.1.13 source GigabitEthernet0/1",
            "no ntp server 172.16.1.111 source GigabitEthernet0/1",
            "no ntp server 172.16.1.19 version 2",
            "no ntp trusted-key 3 - 130",
            "ntp trusted-key 3 - 13",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_ios_ntp_global_replaced_overridden_idempotent(self):
        """both the replaced and overridden states are supported to have same behaviour"""
        self.execute_show_command.return_value = dedent(
            """\
            ntp allow mode control 4
            ntp authenticate
            ntp broadcastdelay 22
            ntp logging
            ntp master 4
            ntp max-associations 34
            ntp maxdistance 3
            ntp mindistance 10
            ntp orphan 4
            ntp panic update
            ntp source Loopback888
            ntp update-calendar
            ntp access-group ipv6 peer preauth_ipv6_acl kod
            ntp access-group peer 2 kod
            ntp authentication-key 2 md5 SomeSecurePassword 7
            ntp peer 172.16.1.11 key 2 minpoll 5 prefer version 2
            ntp peer ip checkPeerDomainIpv4.com prefer
            ntp server ipv6 checkServerDomainIpv6.com
            ntp server 172.16.1.13 source GigabitEthernet0/1
            ntp trusted-key 3 - 13
            ntp trusted-key 21
            """
        )
        set_module_args(
            dict(
                config=dict(
                    access_group=dict(
                        peer=[
                            dict(access_list="2", kod=True),
                            dict(
                                access_list="preauth_ipv6_acl",
                                ipv6=True,
                                kod=True,
                            ),
                        ]
                    ),
                    allow=dict(control=dict(rate_limit=4)),
                    authenticate=True,
                    authentication_keys=[
                        dict(
                            algorithm="md5",
                            encryption=7,
                            id=2,
                            key="SomeSecurePassword",
                        )
                    ],
                    broadcast_delay=22,
                    logging=True,
                    master=dict(stratum=4),
                    max_associations=34,
                    max_distance=3,
                    min_distance=10,
                    orphan=4,
                    panic_update=True,
                    peers=[
                        dict(
                            key_id=2,
                            minpoll=5,
                            peer="172.16.1.11",
                            prefer=True,
                            version=2,
                        ),
                        dict(
                            peer="checkPeerDomainIpv4.com",
                            prefer=True,
                            use_ipv4=True,
                        ),
                    ],
                    servers=[
                        dict(
                            server="172.16.1.13", source="GigabitEthernet0/1"
                        ),
                        dict(
                            server="checkServerDomainIpv6.com", use_ipv6=True
                        ),
                    ],
                    source="Loopback888",
                    trusted_keys=[
                        dict(range_start=21),
                        dict(range_end=13, range_start=3),
                    ],
                    update_calendar=True,
                ),
                state="overridden",
            )
        )
        commands = []
        result = self.execute_module(changed=False)
        self.assertEqual(sorted(result["commands"]), sorted(commands))



#
# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

"""
The iosxr_hostname config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.facts import (
    Facts,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.hostname import (
    HostnameTemplate,
)


class Hostname(ResourceModule):
    """
    The iosxr_hostname config class
    """

    def __init__(self, module):
        super(Hostname, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="hostname",
            tmplt=HostnameTemplate(),
        )
        self.parsers = ["hostname"]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = self.want
        haved = self.have

        if self.state == "deleted":
            wantd = {}

        self._compare(want=wantd, have=haved)

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Hostname network resource.
        """
        self.compare(parsers=self.parsers, want=want, have=have)
